from asset.asset_register import cloud_providers
from asset.base import Asset, AssetColumn, UniqueConstraint
from asset.schema import DbConfig, RamRoleArnCredential
from asset.utils import aliyun_parser_response, get_aliyun_account_id
from typing import List 
from asset.schema import AliyunProfile

import oss2
from alibabacloud_credentials.client import Client
from alibabacloud_credentials.models import Config
from oss2 import CredentialsProvider
from oss2.credentials import Credentials
import datetime


asset_columns = [
    {'name': 'name', 'type': 'str', 'len': 64},
    {'name': 'extranet_endpoint', 'type': 'str', 'len': 128},
    {'name': 'intranet_endpoint', 'type': 'str', 'len': 64},
    {'name': 'location', 'type': 'str', 'len': 64},
    {'name': 'resource_group_id', 'type': 'str', 'len': 64},
    {'name': 'storage_class', 'type': 'str', 'len': 64},
    {'name': 'acl', 'type': 'str', 'len': 64},
    {'name': 'creation_date', 'type': 'datetime'},
]


class CredentialProviderWarpper(CredentialsProvider):
    def __init__(self, client):
        self.client = client

    def get_credentials(self):
        access_key_id = self.client.get_access_key_id()
        access_key_secret = self.client.get_access_key_secret()
        security_token = self.client.get_security_token()
        return Credentials(access_key_id, access_key_secret, security_token)


def timestamp_to_date(timestamp: int) -> datetime.datetime:
    return datetime.datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S')


@cloud_providers.aliyun.register('oss')
class Oss(Asset):
    _platform = 'aliyun'

    _table_name = 'oss'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'name', name='aliyun_oss_'),)

    def _paginate_all_assets(self) -> list:
        assets = []
        auth, endpoint = self._get_client()
        bucket_endpoint = 'https://{location}.aliyuncs.com'

        # get buckets
        print(self.region, endpoint, 22222222222)
        for b in oss2.BucketIterator(oss2.Service(auth, endpoint)):
            b_dict = b.__dict__
            b_dict['creation_date'] = timestamp_to_date(b_dict['creation_date'])
            # get acl
            b_dict['acl'] = oss2.Bucket(auth, endpoint=bucket_endpoint.format(location=b.location), bucket_name=b.name).get_bucket_acl().acl
            assets.append(b_dict)

        return assets

    def _get_assets(self) -> list:
        pass

    def _get_client(self) -> (oss2.ProviderAuth, str):
        config = Config(
            type='ram_role_arn',
            # 从环境变量中获取RAM用户的访问密钥（AccessKey ID和AccessKey Secret）。                  
            access_key_id=self.cred.sts_access_key_id,          
            access_key_secret=self.cred.sts_access_key_secret,  
            # 从环境变量中获取RAM角色的RamRoleArn。
            role_arn=self.cred.role_arn,
            # 填写RAM角色的会话名称。
            role_session_name=self.cred.session_role_name
        )

        cred = Client(config)

        credentials_provider = CredentialProviderWarpper(cred)

        # 使用环境变量中获取的RAM用户的访问密钥和RAM角色的RamRoleArn配置访问凭证。
        auth = oss2.ProviderAuth(credentials_provider)
        endpoint = f'https://oss-{self.region}.aliyuncs.com'
        return auth, endpoint


    def _get_account_id(self):
        return get_aliyun_account_id(cred=self.cred)

    @classmethod
    def load_creds(cls, profile: AliyunProfile) -> List[RamRoleArnCredential]:
        return [
            RamRoleArnCredential(
                profile.ak,
                profile.sk,
                role.arn,
                role.session_name
            ) for role in profile.roles
        ]







# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import HuaweiAsset, AssetColumn, UniqueConstraint
from huaweicloudsdkecs.v2.region.ecs_region import EcsRegion
from huaweicloudsdkevs.v2.region.evs_region import EvsRegion
from huaweicloudsdkeip.v2.region.eip_region import EipRegion
from huaweicloudsdknat.v2.region.nat_region import NatRegion

from huaweicloudsdkecs.v2.ecs_client import EcsClient
from huaweicloudsdkecs.v2.model import ListServersDetailsRequest

from huaweicloudsdkevs.v2.evs_client import EvsClient
from huaweicloudsdkevs.v2.model import ListVolumesRequest

from huaweicloudsdkeip.v2.eip_client import EipClient
from huaweicloudsdkeip.v2.model import ListPublicipsRequest

from huaweicloudsdknat.v2.nat_client import NatClient
from huaweicloudsdknat.v2.model import ListNatGatewaysRequest


ecs_asset_columns = [
    {'name': 'tenant_id', 'type': 'str'},
    {'name': 'metadata', 'type': 'dict'},
    {'name': 'addresses', 'type': 'dict'},
    {'name': 'OS-EXT-STS:task_state', 'type': 'str'},
    {'name': 'OS-DCF:diskConfig', 'type': 'str'},
    {'name': 'OS-EXT-AZ:availability_zone', 'type': 'str'},
    {'name': 'links', 'type': 'dict'},
    {'name': 'OS-EXT-STS:power_state', 'type': 'int'},
    {'name': 'id', 'type': 'str'},
    {'name': 'os-extended-volumes:volumes_attached', 'type': 'dict'},
    {'name': 'OS-EXT-SRV-ATTR:host', 'type': 'str'},
    {'name': 'accessIPv4', 'type': 'str'},
    {'name': 'image', 'type': 'dict'},
    {'name': 'OS-SRV-USG:terminated_at', 'type': 'str'},
    {'name': 'accessIPv6', 'type': 'str'},
    {'name': 'created', 'type': 'str'},
    {'name': 'hostId', 'type': 'str'},
    {'name': 'OS-EXT-SRV-ATTR:hypervisor_hostname', 'type': 'str'},
    {'name': 'flavor', 'type': 'dict'},
    {'name': 'key_name', 'type': 'str'},
    {'name': 'security_groups', 'type': 'dict'},
    {'name': 'config_drive', 'type': 'str'},
    {'name': 'OS-EXT-STS:vm_state', 'type': 'str'},
    {'name': 'user_id', 'type': 'str'},
    {'name': 'OS-EXT-SRV-ATTR:instance_name', 'type': 'str'},
    {'name': 'name', 'type': 'str'},
    {'name': 'OS-SRV-USG:launched_at', 'type': 'str'},
    {'name': 'updated', 'type': 'str'},
    {'name': 'status', 'type': 'str'}
]
evs_asset_columns = [
    {'name': 'attachments', 'type': 'dict'},
    {'name': 'availability_zone', 'type': 'str'},
    {'name': 'bootable', 'type': 'str'},
    {'name': 'created_at', 'type': 'str'},
    {'name': 'id', 'type': 'str'},
    {'name': 'links', 'type': 'dict'},
    {'name': 'metadata', 'type': 'dict'},
    {'name': 'name', 'type': 'str'},
    {'name': 'os-vol-host-attr:host', 'type': 'str'},
    {'name': 'volume_image_metadata', 'type': 'dict'},
    {'name': 'os-vol-tenant-attr:tenant_id', 'type': 'str'},
    {'name': 'replication_status', 'type': 'str'},
    {'name': 'multiattach', 'type': 'str'},
    {'name': 'size', 'type': 'int'},
    {'name': 'status', 'type': 'str'},
    {'name': 'updated_at', 'type': 'str'},
    {'name': 'user_id', 'type': 'str'},
    {'name': 'volume_type', 'type': 'str'},
    {'name': 'service_type', 'type': 'str'},
    {'name': 'wwn', 'type': 'str'}
]
eip_asset_columns = [
    {'name': 'tenant_id', 'type': 'str'},
    {'name': 'bandwidth_name', 'type': 'str'},
    {'name': 'public_ip_address', 'type': 'str'},
    {'name': 'create_time', 'type': 'str'},
    {'name': 'profile', 'type': 'dict'},
    {'name': 'type', 'type': 'str'},
    {'name': 'bandwidth_id', 'type': 'str'},
    {'name': 'bandwidth_size', 'type': 'int'},
    {'name': 'enterprise_project_id', 'type': 'str'},
    {'name': 'ip_version', 'type': 'int'},
    {'name': 'private_ip_address', 'type': 'str'},
    {'name': 'bandwidth_share_type', 'type': 'str'},
    {'name': 'id', 'type': 'str'},
    {'name': 'status', 'type': 'str'},
    {'name': 'port_id', 'type': 'str'},
    {'name': 'public_border_group', 'type': 'str'},
    {'name': 'allow_share_bandwidth_types', 'type': 'dict'}
]
nat_asset_columns = [
    {'name': 'id', 'type': 'str'},
    {'name': 'router_id', 'type': 'str'},
    {'name': 'status', 'type': 'str'},
    {'name': 'description', 'type': 'str'},
    {'name': 'admin_state_up', 'type': 'str'},
    {'name': 'tenant_id', 'type': 'str'},
    {'name': 'created_at', 'type': 'str'},
    {'name': 'spec', 'type': 'str'},
    {'name': 'internal_network_id', 'type': 'str'},
    {'name': 'name', 'type': 'str'},
    {'name': 'enterprise_project_id', 'type': 'str'}
]

ecs_field_document = 'https://support.huaweicloud.com/api-ecs/ecs_05_0002.html'
evs_field_document = 'https://support.huaweicloud.com/api-evs/evs_04_2006.html'
eip_field_document = 'https://support.huaweicloud.com/api-eip/eip_api_0002.html'
nat_field_document = 'https://support.huaweicloud.com/api-natgateway/ListNatGateways.html'


@cloud_providers.huawei.register('ecs')
class Ecs(HuaweiAsset):

    _client = EcsClient
    _region_obj = EcsRegion
    _des = 'list_servers_details'
    _request_obj = ListServersDetailsRequest
    _response_field = 'servers'

    _table_name = 'huawei_ecs'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in ecs_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'id', name='huawei_uc_ecs'),)
    _field_document = ecs_field_document


@cloud_providers.huawei.register('evs')
class Evs(HuaweiAsset):
    _client = EvsClient
    _region_obj = EvsRegion
    _des = 'list_volumes'
    _request_obj = ListVolumesRequest
    _response_field = 'volumes'

    _table_name = 'huawei_evs'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in evs_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'id', name='huawei_uc_evs'),)
    _field_document = evs_field_document


@cloud_providers.huawei.register('eip')
class Eip(HuaweiAsset):
    _client = EipClient
    _region_obj = EipRegion
    _des = 'list_publicips'
    _request_obj = ListPublicipsRequest
    _response_field = 'publicips'
    _request_pars = {'limit': 2000}
    _offset = False

    _table_name = 'huawei_eip'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in eip_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'id', name='huawei_uc_eip'),)
    _field_document = eip_field_document


@cloud_providers.huawei.register('nat')
class Nat(HuaweiAsset):
    _client = NatClient
    _region_obj = NatRegion
    _des = 'list_nat_gateways'
    _request_obj = ListNatGatewaysRequest
    _response_field = 'nat_gateways'
    _request_pars = {'limit': 2000}
    _offset = False

    _table_name = 'huawei_nat'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in nat_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'id', name='huawei_uc_nat'),)
    _field_document = nat_field_document




# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import time


class FetchCtx:
    def __init__(self, fetch):
        self.fetch = fetch
        self.time_now = time.time()

    def __enter__(self):
        self.fetch.logger.info(f'-------{self.fetch.cloud_provider} fetch start -------')

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.fetch.logger.info(f'-------{self.fetch.cloud_provider} fetch end, spend time {time.time() - self.time_now}/s   -------')



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

class BaseCloudAssetException(Exception):
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return f'BaseCloudAssetException: [{self.msg}]'



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import re
import json
import boto3
import datetime
import logging

from tencentcloud.cam.v20190116 import cam_client, models

from aliyunsdksts.request.v20150401 import GetCallerIdentityRequest
from aliyunsdkcore.client import AcsClient

from huaweicloudsdkcore.sdk_response import SdkResponse

from boto3 import client as aws_client
from typing import Tuple, Union

from asset.schema import AwsCredential, STSAssumeRoleCredential, RamRoleArnCredential, HuaweiCredential


_IGNORE_ERROR_MSG = 'EntityNotExist.User.LoginProfile login policy not exists'


def retry(retry_count=5):
    """
    :param retry_count: 重试次数
    :return:
    """
    logger = logging.getLogger('cloud-asset-fetch')

    def decorator(func):
        def handler(*args, **kwargs):

            all_fail, error, response = 0, None, None
            for i in range(retry_count):
                try:
                    if i > 0:
                        logger.info(f'开始重试, 重试次数{i}')
                    response = func(*args, **kwargs)
                    break
                except Exception as e:
                    if _IGNORE_ERROR_MSG in str(e):
                        response = None
                        break

                    all_fail += 1
                    error = e
                    continue

            if all_fail == retry_count:
                logger.info('全部重试失败！')
                raise error

            return response

        return handler
    return decorator


def to_hump_underline(string) -> str:
    return re.sub(r"(?P<key>[A-Z])", r"_\g<key>", string).lower().strip('_')


def register_assets(cloud_provider):
    import importlib
    from asset.asset_register import cloud_providers

    importlib.import_module(f'asset.{cloud_provider}')
    return getattr(cloud_providers, cloud_provider)


def tencent_parser_response(response, response_filed: str) -> Union[list, dict]:
    return json.loads(response.to_json_string())[response_filed]


def aliyun_parser_response(response, response_filed: str, child_response_filed: str = None) -> list:
    if child_response_filed:
        return json.loads(response)[response_filed][child_response_filed]
    return json.loads(response)[response_filed]


def aws_parser_response(
        response: dict, response_filed: str, child_response_filed: str = None) -> Tuple[Union[list, str, dict], str]:
    assets = response.get(response_filed)
    next_token = response.get('NextToken') or response.get('NextMarker')
    if child_response_filed is not None:
        if isinstance(assets, list):
            _assets = []
            for asset in assets:
                _assets += asset[child_response_filed]
            return _assets, next_token
        elif isinstance(assets, dict):
            return assets.get(child_response_filed), next_token
        else:
            raise Exception('')

    if isinstance(assets, dict):
        assets = [assets]

    return assets, next_token


def huawei_parser_response(response: dict, response_filed: str) -> list:
    return response.get(response_filed)


def recursive_dict(_value):
    for key, value in _value.items():
        if isinstance(value, datetime.datetime):
            _value[key] = value.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(value, datetime.date):
            _value[key] = value.strftime('%Y-%m-%d')
        elif isinstance(value, dict):
            _value[key] = recursive_dict(value)
        elif isinstance(value, list):
            _value[key] = recursive_list(value)
        elif isinstance(value, bool):
            _value[key] = str(value)
    return _value


def recursive_list(value):
    _value_list = []
    for index in range(len(value)):
        _value = value[index]
        if isinstance(_value, dict):
            _value_list.append(recursive_dict(_value))
        elif isinstance(_value, list):
            _value_list.append(recursive_list(_value))
        elif isinstance(value, bool):
            _value_list.append(str(_value))
        elif isinstance(value, datetime.datetime):
            _value_list = value.strftime('%Y-%m-%d %H:%M:%S')
        else:
            _value_list.append(_value)
    return _value_list


def recursive_list_huawei_assets(assets: list):
    _assets = []
    for index in range(len(assets)):
        _asset = assets[index]
        if isinstance(_asset, SdkResponse):
            _assets.append(_asset.to_json_object())


def get_tencent_account_id(cred):
    client = cam_client.CamClient(credential=cred, region=None)
    response = client.GetUserAppId(models.GetUserAppIdRequest())
    return response.OwnerUin


def get_aliyun_account_id(cred):
    request = GetCallerIdentityRequest.GetCallerIdentityRequest()
    response = AcsClient(credential=cred).do_action_with_exception(request)
    return json.loads(response)['AccountId']


def get_aws_account_id(cred: AwsCredential):
    response = aws_client('sts', **cred.dict()).get_caller_identity()
    return response['Account']


def get_huawei_account_id(cred: HuaweiCredential):
    from huaweicloudsdkcore.auth.credentials import GlobalCredentials
    from huaweicloudsdkiam.v3.region.iam_region import IamRegion
    from huaweicloudsdkcore.exceptions import exceptions
    from huaweicloudsdkiam.v3.iam_client import IamClient
    from huaweicloudsdkiam.v3.model import KeystoneListAuthDomainsRequest

    credentials = GlobalCredentials(cred.ak, cred.sk)

    client = IamClient.new_builder() \
        .with_credentials(credentials) \
        .with_region(IamRegion.value_of("cn-south-1")) \
        .build()

    try:
        request = KeystoneListAuthDomainsRequest()
        response = client.keystone_list_auth_domains(request).to_dict()
        domains = response.get('domains', [])
        if not domains:
            raise Exception('not find domain id')
        return domains[0].get('id')
    except exceptions.ClientRequestException as e:
        raise e


def aws_assume_role(arn, role_session_name='fetch_asset', duration_seconds=3600) -> AwsCredential:
    response = boto3.client('sts').assume_role(
        RoleArn=arn, RoleSessionName=role_session_name, DurationSeconds=duration_seconds
    )
    role_ak = response['Credentials'].get('AccessKeyId')
    role_sk = response['Credentials'].get('SecretAccessKey')
    role_token = response['Credentials']['SessionToken']
    return AwsCredential(aws_access_key_id=role_ak, aws_secret_access_key=role_sk, aws_session_token=role_token)


def tencent_assume_role(ak, sk, arn, session_name='fetch_asset', duration_seconds=3600):
    return STSAssumeRoleCredential(ak, sk, arn, session_name, duration_seconds)


def aliyun_assume_role(ak, sk, arn, session_name='fetch_asset'):
    return RamRoleArnCredential(ak, sk, arn, session_name)






# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import HuaweiAsset, AssetColumn, UniqueConstraint

from huaweicloudsdkvpc.v3.region.vpc_region import VpcRegion
from huaweicloudsdkvpc.v3.vpc_client import VpcClient
from huaweicloudsdkvpc.v3.model import ListSecurityGroupsRequest, ListSecurityGroupRulesRequest


sg_asset_columns = [
    {'name': 'id', 'type': 'str'},
    {'name': 'name', 'type': 'str'},
    {'name': 'description', 'type': 'str'},
    {'name': 'vpc_id', 'type': 'str'},
    {'name': 'enterprise_project_id', 'type': 'str'},
    {'name': 'security_group_rules', 'type': 'dict'}
]

sg_field_document = 'https://support.huaweicloud.com/api-vpc/vpc_sg01_0003.html'


@cloud_providers.huawei.register('security_groups')
class SecurityGroup(HuaweiAsset):
    _client = VpcClient
    _region_obj = VpcRegion
    _des = 'list_security_groups'
    _request_obj = ListSecurityGroupsRequest
    _response_field = 'security_groups'
    _request_pars = {}
    _offset = False

    _table_name = 'huawei_sg'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in sg_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'id', name='huawei_uc_sg'),)
    _field_document = sg_field_document

    def _paginate_all_assets(self) -> list:
        assets = super(SecurityGroup, self)._paginate_all_assets()
        if not assets:
            return assets

        # find sg detail
        sg_ids = [asset['id'] for asset in assets]
        request = ListSecurityGroupRulesRequest(security_group_id=sg_ids)
        responses = self._describe(
            self.client,
            'list_security_group_rules',
            request,
            'security_group_rules',
            self.parser_response
        ).parser_response()
        for asset in assets:
            asset['security_group_rules'] = list(filter(lambda x: x['security_group_id'] == asset['id'], responses))

        return assets



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import TencentAsset, AssetColumn, UniqueConstraint
from tencentcloud.cvm.v20170312.cvm_client import CvmClient, models

asset_columns = [
    {'name': 'InstanceId', 'type': 'str', 'len': 64},
    {'name': 'Placement', 'type': 'dict'},
    {'name': 'InstanceType', 'type': 'str', 'len': 16},
    {'name': 'CPU', 'type': 'int'},
    {'name': 'Memory', 'type': 'int'},
    {'name': 'RestrictState', 'type': 'str', 'len': 64},
    {'name': 'InstanceName', 'type': 'str', 'len': 64},
    {'name': 'InstanceChargeType', 'type': 'str', 'len': 64},
    {'name': 'SystemDisk', 'type': 'dict'},
    {'name': 'DataDisks', 'type': 'dict'},
    {'name': 'PrivateIpAddresses', 'type': 'dict'},
    {'name': 'PublicIpAddresses', 'type': 'dict'},
    {'name': 'InternetAccessible', 'type': 'dict'},
    {'name': 'VirtualPrivateCloud', 'type': 'dict'},
    {'name': 'ImageId', 'type': 'str', 'len': 64},
    {'name': 'RenewFlag', 'type': 'str', 'len': 64},
    {'name': 'CreatedTime', 'type': 'str', 'len': 64},
    {'name': 'ExpiredTime', 'type': 'str', 'len': 64},
    {'name': 'OsName', 'type': 'str', 'len': 64},
    {'name': 'SecurityGroupIds', 'type': 'dict'},
    {'name': 'LoginSettings', 'type': 'dict'},
    {'name': 'InstanceState', 'type': 'str', 'len': 64},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'LicenseType', 'type': 'str', 'len': 64},
    {'name': 'CamRoleName', 'type': 'str', 'len': 128}
]

cvm_field_document = 'https://cloud.tencent.com/document/api/213/15728'


@cloud_providers.tencent.register('cvm')
class Cvm(TencentAsset):
    _des_request_func = 'DescribeInstances'
    _des_request = models.DescribeInstancesRequest()
    _response_field = 'InstanceSet'

    _table_name = 'tencent_cvm'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'instance_id', name='tencent_instance'),)
    _field_document = cvm_field_document

    def _get_client(self):
        return CvmClient(self.cred, self.region)




# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import importlib

from asset.asset_register import cloud_providers
from asset.schema import DbConfig, TencentProfile, AliyunProfile, AwsProfile, HuaweiProfile
from asset.ctx import FetchCtx

from typing import Union
from logging.handlers import TimedRotatingFileHandler


_default_log_dir_path = './'


class Fetch:
    logger = logging.getLogger('cloud-asset-fetch')

    def __init__(
            self,
            cloud_provider,
            profile: Union[TencentProfile, AliyunProfile, AwsProfile, HuaweiProfile],
            assets: Union[list, str],
            regions,
            log_dir_path=_default_log_dir_path,
            dbconfig=DbConfig()
    ):
        self.cloud_provider = cloud_provider
        self._register_assets = self.register_assets(cloud_provider=self.cloud_provider).assets  # type: dict

        self.profile = profile
        self.assets = list(self._register_assets.keys()) if assets == 'all' else assets
        self.regions = regions
        self.log_dir_path = log_dir_path
        self.dbconfig = dbconfig

        self._setup_logger()

    def fetch(self):
        with FetchCtx(self):
            for asset_name in self.assets:
                asset_obj = self._register_assets[asset_name]
                for cred in asset_obj.load_creds(self.profile):
                    for region in self.regions:
                        try:
                            asset = asset_obj(cred, region=region, dbconfig=self.dbconfig)
                            self.logger.info(
                                f'asset: {asset_name} ----region: {region} ----account_id: {asset.account_id}----start')
                            asset.fetch()
                        except Exception as e:
                            import traceback
                            self.logger.info(
                                f'asset: {asset_name} ----region: {region} ----account_id: {asset.account_id}----fail----error: {traceback.format_exc()}'
                            )
                            continue
                        self.logger.info(
                            f'asset: {asset_name} ----region: {region} ----account_id: {asset.account_id}----end')

    @classmethod
    def register_assets(cls, cloud_provider):
        importlib.import_module(f'asset.{cloud_provider}')
        return getattr(cloud_providers, cloud_provider)

    def _setup_logger(self):
        dir_path = os.path.join(self.log_dir_path, 'fetch_log')
        if not os.path.exists(dir_path):
            os.mkdir(dir_path)
        file_handler = TimedRotatingFileHandler(
            filename=os.path.join(dir_path, 'fetch.log'), when="MIDNIGHT", interval=1, backupCount=7, encoding='utf-8'
        )
        file_handler.setLevel(logging.INFO)
        file_handler.setFormatter(logging.Formatter("%(asctime)s: %(name)s:%(levelname)s:%(message)s"))

        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.INFO)

        self.logger.addHandler(file_handler)
        self.logger.addHandler(console_handler)
        self.logger.setLevel(logging.INFO)






# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import AwsAsset, AssetColumn, UniqueConstraint


db_instances_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/rds.html#RDS.Client.describe_db_instances'
db_clusters_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/rds.html#RDS.Client.describe_db_clusters'

db_instances_asset_columns = [
    {'name': 'DBInstanceIdentifier', 'type': 'str'},
    {'name': 'DBInstanceClass', 'type': 'str'},
    {'name': 'Engine', 'type': 'str'},
    {'name': 'DBInstanceStatus', 'type': 'str'},
    {'name': 'AutomaticRestartTime', 'type': 'datetime'},
    {'name': 'MasterUsername', 'type': 'str'},
    {'name': 'DBName', 'type': 'str'},
    {'name': 'Endpoint', 'type': 'dict'},
    {'name': 'AllocatedStorage', 'type': 'int'},
    {'name': 'InstanceCreateTime', 'type': 'datetime'},
    {'name': 'PreferredBackupWindow', 'type': 'str'},
    {'name': 'BackupRetentionPeriod', 'type': 'int'},
    {'name': 'DBSecurityGroups', 'type': 'dict'},
    {'name': 'VpcSecurityGroups', 'type': 'dict'},
    {'name': 'DBParameterGroups', 'type': 'dict'},
    {'name': 'AvailabilityZone', 'type': 'str'},
    {'name': 'DBSubnetGroup', 'type': 'dict'},
    {'name': 'PreferredMaintenanceWindow', 'type': 'str'},
    {'name': 'PendingModifiedValues', 'type': 'dict'},
    {'name': 'LatestRestorableTime', 'type': 'datetime'},
    {'name': 'MultiAZ', 'type': 'bool'},
    {'name': 'EngineVersion', 'type': 'str'},
    {'name': 'AutoMinorVersionUpgrade', 'type': 'bool'},
    {'name': 'ReadReplicaSourceDBInstanceIdentifier', 'type': 'str'},
    {'name': 'ReadReplicaDBInstanceIdentifiers', 'type': 'dict'},
    {'name': 'ReplicaMode', 'type': 'str'},
    {'name': 'LicenseModel', 'type': 'str'},
    {'name': 'Iops', 'type': 'int'},
    {'name': 'OptionGroupMemberships', 'type': 'dict'},
    {'name': 'CharacterSetName', 'type': 'str'},
    {'name': 'NcharCharacterSetName', 'type': 'str'},
    {'name': 'SecondaryAvailabilityZone', 'type': 'str'},
    {'name': 'PubliclyAccessible', 'type': 'bool'},
    {'name': 'StatusInfos', 'type': 'dict'},
    {'name': 'StorageType', 'type': 'str'},
    {'name': 'TdeCredentialArn', 'type': 'str'},
    {'name': 'DbInstancePort', 'type': 'int'},
    {'name': 'DBClusterIdentifier', 'type': 'str'},
    {'name': 'StorageEncrypted', 'type': 'bool'},
    {'name': 'KmsKeyId', 'type': 'str'},
    {'name': 'DbiResourceId', 'type': 'str'},
    {'name': 'CACertificateIdentifier', 'type': 'str'},
    {'name': 'DomainMemberships', 'type': 'dict'},
    {'name': 'CopyTagsToSnapshot', 'type': 'bool'},
    {'name': 'MonitoringInterval', 'type': 'int'},
    {'name': 'EnhancedMonitoringResourceArn', 'type': 'str'},
    {'name': 'MonitoringRoleArn', 'type': 'str'},
    {'name': 'PromotionTier', 'type': 'int'},
    {'name': 'DBInstanceArn', 'type': 'str'},
    {'name': 'Timezone', 'type': 'str'},
    {'name': 'IAMDatabaseAuthenticationEnabled', 'type': 'bool'},
    {'name': 'PerformanceInsightsEnabled', 'type': 'bool'},
    {'name': 'PerformanceInsightsKMSKeyId', 'type': 'str'},
    {'name': 'PerformanceInsightsRetentionPeriod', 'type': 'int'},
    {'name': 'EnabledCloudwatchLogsExports', 'type': 'dict'},
    {'name': 'ProcessorFeatures', 'type': 'dict'},
    {'name': 'DeletionProtection', 'type': 'bool'},
    {'name': 'AssociatedRoles', 'type': 'dict'},
    {'name': 'ListenerEndpoint', 'type': 'dict'},
    {'name': 'MaxAllocatedStorage', 'type': 'int'},
    {'name': 'TagList', 'type': 'dict'},
    {'name': 'DBInstanceAutomatedBackupsReplications', 'type': 'dict'},
    {'name': 'CustomerOwnedIpEnabled', 'type': 'bool'},
    {'name': 'AwsBackupRecoveryPointArn', 'type': 'str'},
    {'name': 'ActivityStreamStatus', 'type': 'str'},
    {'name': 'ActivityStreamKmsKeyId', 'type': 'str'},
    {'name': 'ActivityStreamKinesisStreamName', 'type': 'str'},
    {'name': 'ActivityStreamMode', 'type': 'str'},
    {'name': 'ActivityStreamEngineNativeAuditFieldsIncluded', 'type': 'bool'},
    {'name': 'AutomationMode', 'type': 'str'},
    {'name': 'ResumeFullAutomationModeTime', 'type': 'datetime'},
    {'name': 'CustomIamInstanceProfile', 'type': 'str'},
    {'name': 'BackupTarget', 'type': 'str'},
    {'name': 'NetworkType', 'type': 'str'},
    {'name': 'ActivityStreamPolicyStatus', 'type': 'str'},
    {'name': 'StorageThroughput', 'type': 'int'},
    {'name': 'DBSystemId', 'type': 'str'},
    {'name': 'MasterUserSecret', 'type': 'dict'},
    {'name': 'CertificateDetails', 'type': 'dict'}
]
db_clusters_asset_columns = [
    {'name': 'AllocatedStorage', 'type': 'str'},
    {'name': 'AvailabilityZones', 'type': 'dict'},
    {'name': 'BackupRetentionPeriod', 'type': 'int'},
    {'name': 'CharacterSetName', 'type': 'str'},
    {'name': 'DatabaseName', 'type': 'str'},
    {'name': 'DBClusterIdentifier', 'type': 'str'},
    {'name': 'DBClusterParameterGroup', 'type': 'str'},
    {'name': 'DBSubnetGroup', 'type': 'str'},
    {'name': 'Status', 'type': 'str'},
    {'name': 'AutomaticRestartTime', 'type': 'datetime'},
    {'name': 'PercentProgress', 'type': 'str'},
    {'name': 'EarliestRestorableTime', 'type': 'datetime'},
    {'name': 'Endpoint', 'type': 'str'},
    {'name': 'ReaderEndpoint', 'type': 'str'},
    {'name': 'CustomEndpoints', 'type': 'dict'},
    {'name': 'MultiAZ', 'type': 'bool'},
    {'name': 'Engine', 'type': 'str'},
    {'name': 'EngineVersion', 'type': 'str'},
    {'name': 'LatestRestorableTime', 'type': 'dict'},
    {'name': 'Port', 'type': 'int'},
    {'name': 'MasterUsername', 'type': 'str'},
    {'name': 'DBClusterOptionGroupMemberships', 'type': 'dict'},
    {'name': 'PreferredBackupWindow', 'type': 'str'},
    {'name': 'PreferredMaintenanceWindow', 'type': 'str'},
    {'name': 'ReplicationSourceIdentifier', 'type': 'str'},
    {'name': 'ReadReplicaIdentifiers', 'type': 'dict'},
    {'name': 'DBClusterMembers', 'type': 'dict'},
    {'name': 'VpcSecurityGroups', 'type': 'dict'},
    {'name': 'HostedZoneId', 'type': 'str'},
    {'name': 'StorageEncrypted', 'type': 'bool'},
    {'name': 'KmsKeyId', 'type': 'str'},
    {'name': 'DbClusterResourceId', 'type': 'bool'},
    {'name': 'DBClusterArn', 'type': 'str'},
    {'name': 'AssociatedRoles', 'type': 'dict'},
    {'name': 'IAMDatabaseAuthenticationEnabled', 'type': 'bool'},
    {'name': 'CloneGroupId', 'type': 'str'},
    {'name': 'ClusterCreateTime', 'type': 'datetime'},
    {'name': 'EarliestBacktrackTime', 'type': 'datetime'},
    {'name': 'BacktrackWindow', 'type': 'int'},
    {'name': 'BacktrackConsumedChangeRecords', 'type': 'int'},
    {'name': 'EnabledCloudwatchLogsExports', 'type': 'dict'},
    {'name': 'Capacity', 'type': 'int'},
    {'name': 'EngineMode', 'type': 'str'},
    {'name': 'ScalingConfigurationInfo', 'type': 'dict'},
    {'name': 'DeletionProtection', 'type': 'bool'},
    {'name': 'HttpEndpointEnabled', 'type': 'bool'},
    {'name': 'ActivityStreamMode', 'type': 'str'},
    {'name': 'ActivityStreamStatus', 'type': 'str'},
    {'name': 'ActivityStreamKmsKeyId', 'type': 'str'},
    {'name': 'ActivityStreamKinesisStreamName', 'type': 'str'},
    {'name': 'CopyTagsToSnapshot', 'type': 'bool'},
    {'name': 'CrossAccountClone', 'type': 'bool'},
    {'name': 'DomainMemberships', 'type': 'dict'},
    {'name': 'TagList', 'type': 'dict'},
    {'name': 'GlobalWriteForwardingStatus', 'type': 'str'},
    {'name': 'GlobalWriteForwardingRequested', 'type': 'bool'},
    {'name': 'PendingModifiedValues', 'type': 'dict'},
    {'name': 'DBClusterInstanceClass', 'type': 'str'},
    {'name': 'StorageType', 'type': 'str'},
    {'name': 'Iops', 'type': 'int'},
    {'name': 'PubliclyAccessible', 'type': 'bool'},
    {'name': 'AutoMinorVersionUpgrade', 'type': 'bool'},
    {'name': 'MonitoringInterval', 'type': 'int'},
    {'name': 'MonitoringRoleArn', 'type': 'str'},
    {'name': 'PerformanceInsightsEnabled', 'type': 'bool'},
    {'name': 'PerformanceInsightsKMSKeyId', 'type': 'str'},
    {'name': 'PerformanceInsightsRetentionPeriod', 'type': 'int'},
    {'name': 'ServerlessV2ScalingConfiguration', 'type': 'dict'},
    {'name': 'NetworkType', 'type': 'str'},
    {'name': 'DBSystemId', 'type': 'str'},
    {'name': 'MasterUserSecret', 'type': 'dict'}
]


@cloud_providers.aws.register('rds_db_instances')
class DbInstances(AwsAsset):
    _client_name = 'rds'
    _des_request = 'describe_db_instances'
    _response_field = 'DBInstances'
    _des_request_kwargs: dict = {'MaxRecords': 100}

    _table_name = 'aws_rds_db_instances'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in db_instances_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'd_b_instance_identifier', name='aws_uc_rds_db_i'),)
    _field_document = db_instances_field_document


@cloud_providers.aws.register('rds_db_clusters')
class DbClusters(AwsAsset):
    _client_name = 'rds'
    _des_request = 'describe_db_clusters'
    _response_field = 'DBClusters'
    _des_request_kwargs: dict = {'MaxRecords': 100}

    _table_name = 'aws_rds_db_clusters'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in db_clusters_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'd_b_cluster_identifier', name='aws_uc_rds_db_c'),)
    _field_document = db_clusters_field_document




# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0
import copy
import datetime
from asset.asset_register import cloud_providers
from asset.base import TencentAsset, AssetColumn, UniqueConstraint
from asset.conf import today, yesterday

from tencentcloud.monitor.v20180724.monitor_client import MonitorClient, models

ccu_queries = {
    'Namespace': 'QCE/CVM',
    'MetricName': 'CPUUsage',
    'Period': 3600,
    'Instances': [],
    'StartTime': datetime.datetime(yesterday.year, yesterday.month, yesterday.day).isoformat() + '+08:00',
    'EndTime': datetime.datetime(today.year, today.month, today.day).isoformat() + '+08:00',
}
cvm_ccu_asset_columns = [
    {'name': 'InstanceId', 'type': 'str', 'len': 64},
    {'name': 'MetricName', 'type': 'str', 'len': 16},
    {'name': 'Period', 'type': 'int'},
    {'name': 'Timestamp', 'type': 'datetime'},
    {'name': 'MetricValue', 'type': 'float'}
]

get_metric_field_document = 'https://cloud.tencent.com/document/api/248/31014'


@cloud_providers.tencent.register('cvm_cpu_usage')
class CvmCpuUsage(TencentAsset):
    _des_request_func = 'GetMonitorData'
    _des_request = models.GetMonitorDataRequest()
    _response_field = 'DataPoints'

    _table_name = 'tencent_cvm_cpu_usage'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in cvm_ccu_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'timestamp', 'instance_id', name='tencent_instance_ccu'), )
    _field_document = get_metric_field_document

    def _get_client(self):
        return MonitorClient(self.cred, self.region)

    def _paginate_all_assets(self):
        """初始化self._des_request的参数↓"""
        _des_request = copy.deepcopy(self._des_request)
        for key, value in ccu_queries.items():
            setattr(_des_request, key, value)

        from asset.tencent.cvm import Cvm
        assets = []
        instances = Cvm(self.cred, region=self.region, dbconfig=self.dbconfig).paginate_all_assets
        pagesize = 50

        for index in range(len(instances) // pagesize + 1):
            _instances = instances[index * pagesize: index * pagesize + pagesize]
            if not _instances:
                continue
            _des_request.Instances = [
                {'Dimensions': [{'Name': 'InstanceId', 'Value': _instance['InstanceId']}]} for _instance in _instances]
            results = self._describe(
                self.client, self._des_request_func, _des_request, self._response_field).parser_response()
            for result in results:
                count = 0
                for _time_stamp in result['Timestamps']:
                    assets.append(
                        {
                            'InstanceId': result['Dimensions'][0]['Value'],
                            'MetricName': 'CpuUsage',
                            'Period': 3600,
                            'Timestamp': datetime.datetime.fromtimestamp(_time_stamp),
                            'MetricValue': result['Values'][count]
                        }
                    )
                    count += 1

        return sorted(assets, key=lambda x: x['Timestamp'])








# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import copy

from asset.asset_register import cloud_providers
from asset.base import AwsAsset, AssetColumn, UniqueConstraint

elbv2_filed_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/elbv2.html#ElasticLoadBalancingv2.Client.describe_load_balancers'
elbv2_listener_filed_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/elbv2.html#ElasticLoadBalancingv2.Client.describe_listeners'
elbv2_tgs_filed_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/elbv2.html#ElasticLoadBalancingv2.Client.describe_target_groups'
elbv2_tgs_health_filed_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/elbv2.html#ElasticLoadBalancingv2.Client.describe_target_health'

elbv2_asset_columns = [
    {'name': 'LoadBalancerArn', 'type': 'str'},
    {'name': 'DNSName', 'type': 'str'},
    {'name': 'CanonicalHostedZoneId', 'type': 'str'},
    {'name': 'CreatedTime', 'type': 'date'},
    {'name': 'LoadBalancerName', 'type': 'str'},
    {'name': 'Scheme', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'State', 'type': 'dict'},
    {'name': 'Type', 'type': 'str'},
    {'name': 'AvailabilityZones', 'type': 'dict'},
    {'name': 'SecurityGroups', 'type': 'dict'},
    {'name': 'IpAddressType', 'type': 'str'},
    {'name': 'CustomerOwnedIpv4Pool', 'type': 'str'},
    {'name': 'Tags', 'type': 'list'},
]
elbv2_listener_asset_columns = [
    {'name': 'ListenerArn', 'type': 'str'},
    {'name': 'LoadBalancerArn', 'type': 'str'},
    {'name': 'Port', 'type': 'int'},
    {'name': 'Protocol', 'type': 'str'},
    {'name': 'Certificates', 'type': 'dict'},
    {'name': 'SslPolicy', 'type': 'str'},
    {'name': 'DefaultActions', 'type': 'dict'},
    {'name': 'AlpnPolicy', 'type': 'dict'}
]

elbv2_tgs_asset_columns = [
    {'name': 'TargetGroupArn', 'type': 'str'},
    {'name': 'TargetGroupName', 'type': 'str'},
    {'name': 'Protocol', 'type': 'str'},
    {'name': 'Port', 'type': 'int'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'HealthCheckProtocol', 'type': 'str'},
    {'name': 'HealthCheckPort', 'type': 'str'},
    {'name': 'HealthCheckEnabled', 'type': 'str'},
    {'name': 'HealthCheckIntervalSeconds', 'type': 'int'},
    {'name': 'HealthCheckTimeoutSeconds', 'type': 'int'},
    {'name': 'HealthyThresholdCount', 'type': 'int'},
    {'name': 'UnhealthyThresholdCount', 'type': 'int'},
    {'name': 'HealthCheckPath', 'type': 'str'},
    {'name': 'Matcher', 'type': 'dict'},
    {'name': 'LoadBalancerArns', 'type': 'dict'},
    {'name': 'TargetType', 'type': 'str'},
    {'name': 'ProtocolVersion', 'type': 'str'},
    {'name': 'IpAddressType', 'type': 'str'}
]
elbv2_tas_health_asset_columns = [
    # {'name': 'Target', 'type': 'dict'},
    {'name': 'Id', 'type': 'str'},
    {'name': 'Port', 'type': 'int'},
    {'name': 'HealthCheckPort', 'type': 'str'},
    {'name': 'TargetHealth', 'type': 'dict'},
    {'name': 'TargetGroupArn', 'type': 'str'}
]


@cloud_providers.aws.register('elb_v2')
class ElbV2(AwsAsset):
    _client_name = 'elbv2'
    _des_request = 'describe_load_balancers'
    _response_field = 'LoadBalancers'
    _des_request_kwargs: dict = {'PageSize': 100}

    _table_name = 'aws_elb_v2'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in elbv2_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'load_balancer_arn', name='aws_uc_elbv2'),)
    _field_document = elbv2_filed_document

    def _paginate_all_assets(self) -> list:
        tags = []
        _des_request_kwargs = copy.deepcopy(self._des_request_kwargs)
        _assets, _ = self._describe(
            self.client,
            self._des_request,
            self._response_field,
            self._child_response_filed,
            des_request_kwargs=_des_request_kwargs,
            parser_response_func=self.parser_response
        ).parser_response()

        for index in range(len(_assets) // 20 + 1):
            """get tags"""
            elb_list = _assets[index: 20: index * 20 + 20]
            if elb_list:
                break
            _tags, _ = self._describe(
                self.client,
                'describe_tags',
                'TagDescriptions',
                des_request_kwargs={'ResourceArns': [elb['LoadBalancerArn'] for elb in elb_list]},
                parser_response_func=self.parser_response
            )
            tags += _tags
        for asset in _assets:
            try:
                tag = filter(lambda x: x['LoadBalancerArn'] == asset['LoadBalancerArn'], tags).__next__()
            except Exception as e:
                tag = []
            asset.update({'Tags': tag})
        return _assets


@cloud_providers.aws.register('elb_v2_listeners')
class ElbV2Listeners(AwsAsset):
    _client_name = 'elbv2'
    _des_request = 'describe_listeners'
    _response_field = 'Listeners'
    _des_request_kwargs: dict = {'PageSize': 50}
    _next_type = 'Marker'

    _table_name = 'aws_elb_v2_listener'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in elbv2_listener_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'listener_arn', name='aws_uc_elbv2_listener'),)
    _field_document = elbv2_listener_filed_document

    def _paginate_all_assets(self) -> list:
        assets = []
        elb_list = ElbV2(self.cred, self.region, dbconfig=self.dbconfig).paginate_all_assets
        _des_request_kwargs = copy.deepcopy(self._des_request_kwargs)

        for elb in elb_list:
            elb_arn = elb['LoadBalancerArn']
            while True:
                _des_request_kwargs.update({'LoadBalancerArn': elb_arn})
                _assets, next_token = self._describe(
                    self.client,
                    self._des_request,
                    self._response_field,
                    self._child_response_filed,
                    des_request_kwargs=_des_request_kwargs,
                    parser_response_func=self.parser_response
                ).parser_response()
                assets += _assets
                if next_token is None:
                    break
                _des_request_kwargs.update({self._next_type: next_token})
        return assets


@cloud_providers.aws.register('elb_v2_target_groups')
class ElbV2TargetGroups(AwsAsset):
    _client_name = 'elbv2'
    _des_request = 'describe_target_groups'
    _response_field = 'TargetGroups'
    _des_request_kwargs: dict = {'PageSize': 50}
    _next_type = 'Marker'

    _table_name = 'aws_elb_v2_target_groups'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in elbv2_tgs_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'target_group_arn', name='aws_uc_elbv2_target_groups'),)
    _field_document = elbv2_tgs_filed_document


@cloud_providers.aws.register('elb_v2_target_group_health')
class ElbV2TargetGroupHealth(AwsAsset):
    _client_name = 'elbv2'
    _des_request = 'describe_target_health'
    _response_field = 'TargetHealthDescriptions'
    _des_request_kwargs: dict = {'PageSize': 50}
    _next_type = 'Marker'

    _table_name = 'aws_elb_v2_target_group_health'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in elbv2_tas_health_asset_columns]
    _table_args = (
        UniqueConstraint('account_id', 'record_date', 'id', 'target_group_arn',  name='aws_uc_elbv2_tg_health'),)
    _field_document = elbv2_tgs_health_filed_document

    def _paginate_all_assets(self) -> list:
        assets = []
        target_groups = ElbV2TargetGroups(self.cred, region=self.region, dbconfig=self.dbconfig).paginate_all_assets
        for target_group in target_groups:
            arn = target_group['TargetGroupArn']
            _assets, _ = self._describe(
                self.client,
                self._des_request,
                self._response_field,
                self._child_response_filed,
                des_request_kwargs={'TargetGroupArn': arn},
                parser_response_func=self.parser_response
            ).parser_response()
            for _asset in _assets:
                _asset.update(_asset.pop('Target'))
                _asset.update({'TargetGroupArn': arn})
            assets += _assets
        return assets



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import AwsAsset, AssetColumn, UniqueConstraint

ec2_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_instances'
nat_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_nat_gateways'
vpc_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_vpcs'
subnet_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_subnets'
sg_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_subnets'
nt_filed_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_network_interfaces'
eip_filed_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2/client/describe_addresses.html'

ec2_asset_columns = [
    {'name': 'AmiLaunchIndex ', 'type': 'int'},
    {'name': 'ImageId', 'type': 'str', 'len': 64},
    {'name': 'InstanceId', 'type': 'str', 'len': 64},
    {'name': 'InstanceType', 'type': 'str', 'len': 16},
    {'name': 'KernelId', 'type': 'str', 'len': 64},
    {'name': 'KeyName', 'type': 'str', 'len': 32},
    {'name': 'LaunchTime', 'type': 'date'},
    {'name': 'Monitoring', 'type': 'dict'},
    {'name': 'Placement', 'type': 'dict'},
    {'name': 'PlatformDetails', 'type': 'str', 'len': 16},
    {'name': 'PrivateDnsName', 'type': 'str', 'len': 128},
    {'name': 'PrivateIpAddress', 'type': 'str', 'len': 32},
    {'name': 'ProductCodes', 'type': 'dict'},
    {'name': 'PublicDnsName', 'type': 'str', 'len': 128},
    {'name': 'PublicIpAddress', 'type': 'str', 'len': 128},
    {'name': 'RamdiskId', 'type': 'str', 'len': 128},
    {'name': 'State', 'type': 'dict'},
    {'name': 'StateTransitionReason', 'type': 'str', 'len': 128},
    {'name': 'SubnetId', 'type': 'str', 'len': 64},
    {'name': 'VpcId', 'type': 'str', 'len': 64},
    {'name': 'Architecture', 'type': 'str', 'len': 32},
    {'name': 'BlockDeviceMappings', 'type': 'dict'},
    {'name': 'ClientToken', 'type': 'str', 'len': 128},
    {'name': 'EbsOptimized', 'type': 'bool'},
    {'name': 'EnaSupport', 'type': 'bool'},
    {'name': 'Hypervisor', 'type': 'str', 'len': 8},
    {'name': 'IamInstanceProfile', 'type': 'dict'},
    {'name': 'InstanceLifecycle', 'type': 'str', 'len': 8},
    {'name': 'ElasticGpuAssociations', 'type': 'dict'},
    {'name': 'ElasticInferenceAcceleratorAssociations', 'type': 'dict'},
    {'name': 'NetworkInterfaces', 'type': 'dict'},
    {'name': 'OutpostArn', 'type': 'str', 'len': 64},
    {'name': 'RootDeviceName', 'type': 'str', 'len': 64},
    {'name': 'RootDeviceType', 'type': 'str', 'len': 16},
    {'name': 'SecurityGroups', 'type': 'dict'},
    {'name': 'SourceDestCheck', 'type': 'bool'},
    {'name': 'SpotInstanceRequestId', 'type': 'str', 'len': 64},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'VirtualizationType', 'type': 'str', 'len': 32},
    {'name': 'HibernationOptions', 'type': 'dict'},
    {'name': 'Licenses', 'type': 'dict'},
    {'name': 'Ipv6Address', 'type': 'str', 'len': 64},
]
nat_asset_columns = [
    {'name': 'CreateTime', 'type': 'datetime'},
    {'name': 'DeleteTime', 'type': 'datetime'},
    {'name': 'FailureCode', 'type': 'str'},
    {'name': 'FailureMessage', 'type': 'str'},
    {'name': 'NatGatewayAddresses', 'type': 'dict'},
    {'name': 'NatGatewayId', 'type': 'str'},
    {'name': 'ProvisionedBandwidth', 'type': 'dict'},
    {'name': 'State', 'type': 'str'},
    {'name': 'SubnetId', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'ConnectivityType', 'type': 'str'}
]
vpc_asset_columns = [
    {'name': 'CidrBlock', 'type': 'str'},
    {'name': 'DhcpOptionsId', 'type': 'str'},
    {'name': 'State', 'type': 'str', 'length': 16},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'OwnerId', 'type': 'str'},
    {'name': 'InstanceTenancy', 'type': 'str', 'length': 16},
    {'name': 'Ipv6CidrBlockAssociationSet', 'type': 'dict'},
    {'name': 'CidrBlockAssociationSet', 'type': 'dict'},
    {'name': 'IsDefault', 'type': 'bool'},
    {'name': 'Tags', 'type': 'dict'}
]
subnet_asset_columns = [
    {'name': 'AvailabilityZone', 'type': 'str'},
    {'name': 'AvailabilityZoneId', 'type': 'str'},
    {'name': 'AvailableIpAddressCount', 'type': 'int'},
    {'name': 'CidrBlock', 'type': 'str'},
    {'name': 'DefaultForAz', 'type': 'bool'},
    {'name': 'EnableLniAtDeviceIndex', 'type': 'int'},
    {'name': 'MapPublicIpOnLaunch', 'type': 'bool'},
    {'name': 'MapCustomerOwnedIpOnLaunch', 'type': 'bool'},
    {'name': 'CustomerOwnedIpv4Pool', 'type': 'str'},
    {'name': 'SubnetId', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'OwnerId', 'type': 'str'},
    {'name': 'AssignIpv6AddressOnCreation', 'type': 'bool'},
    {'name': 'Ipv6CidrBlockAssociationSet', 'type': 'dict'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'SubnetArn', 'type': 'str'},
    {'name': 'OutpostArn', 'type': 'str'},
    {'name': 'EnableDns64', 'type': 'bool'},
    {'name': 'Ipv6Native', 'type': 'bool'},
    {'name': 'PrivateDnsNameOptionsOnLaunch', 'type': 'dict'}
]
sg_asset_columns = [
    {'name': 'Description', 'type': 'str'},
    {'name': 'GroupName', 'type': 'str'},
    {'name': 'IpPermissions', 'type': 'dict'},
    {'name': 'OwnerId', 'type': 'str'},
    {'name': 'GroupId', 'type': 'str'},
    {'name': 'IpPermissionsEgress', 'type': 'dict'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'VpcId', 'type': 'str'}
]
nt_asset_columns = [
    {'name': 'Association', 'type': 'dict'},
    {'name': 'Attachment', 'type': 'dict'},
    {'name': 'AvailabilityZone', 'type': 'str'},
    {'name': 'Description', 'type': 'text'},
    {'name': 'Groups', 'type': 'dict'},
    {'name': 'InterfaceType', 'type': 'str'},
    {'name': 'Ipv6Addresses', 'type': 'dict'},
    {'name': 'MacAddress', 'type': 'str'},
    {'name': 'NetworkInterfaceId', 'type': 'str'},
    {'name': 'OutpostArn', 'type': 'str'},
    {'name': 'OwnerId', 'type': 'str'},
    {'name': 'PrivateDnsName', 'type': 'str'},
    {'name': 'PrivateIpAddress', 'type': 'str'},
    {'name': 'PrivateIpAddresses', 'type': 'dict'},
    {'name': 'Ipv4Prefixes', 'type': 'dict'},
    {'name': 'Ipv6Prefixes', 'type': 'dict'},
    {'name': 'RequesterId', 'type': 'str'},
    {'name': 'RequesterManaged', 'type': 'bool'},
    {'name': 'SourceDestCheck', 'type': 'bool'},
    {'name': 'Status', 'type': 'str', 'length': 16},
    {'name': 'SubnetId', 'type': 'str'},
    {'name': 'TagSet', 'type': 'dict'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'DenyAllIgwTraffic', 'type': 'bool'},
    {'name': 'Ipv6Native', 'type': 'bool'},
    {'name': 'Ipv6Address', 'type': 'str'}
]
eip_assets_columns = [
    {'name': 'InstanceId', 'type': 'str'},
    {'name': 'PublicIp', 'type': 'str'},
    {'name': 'AllocationId', 'type': 'str'},
    {'name': 'AssociationId', 'type': 'str'},
    {'name': 'Domain', 'type': 'str'},
    {'name': 'NetworkInterfaceId', 'type': 'str'},
    {'name': 'NetworkInterfaceOwnerId', 'type': 'str'},
    {'name': 'PrivateIpAddress', 'type': 'str'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'PublicIpv4Pool', 'type': 'str'},
    {'name': 'NetworkBorderGroup', 'type': 'str'},
    {'name': 'CustomerOwnedIp', 'type': 'str'},
    {'name': 'CustomerOwnedIpv4Pool', 'type': 'str'},
    {'name': 'CarrierIp', 'type': 'str'}
]


@cloud_providers.aws.register('ec2')
class Ec2(AwsAsset):
    _client_name = 'ec2'
    _des_request = 'describe_instances'
    _response_field = 'Reservations'
    _child_response_filed = 'Instances'

    _table_name = 'aws_ec2'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in ec2_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'instance_id', name='aws_uc_ec2'),)
    _field_document = ec2_field_document


@cloud_providers.aws.register('nat_gateways')
class NatGateways(AwsAsset):
    _client_name = 'ec2'
    _des_request = 'describe_nat_gateways'
    _response_field = 'NatGateways'

    _table_name = 'aws_nat_gateways'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in nat_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'nat_gateway_id', name='aws_uc_nat_g'),)
    _field_document = nat_field_document


@cloud_providers.aws.register('vpc')
class Vpc(AwsAsset):
    _client_name = 'ec2'
    _des_request = 'describe_vpcs'
    _response_field = 'Vpcs'

    _table_name = 'aws_vpcs'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in vpc_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'vpc_id', name='aws_uc_vpc'),)
    _field_document = vpc_field_document


@cloud_providers.aws.register('subnets')
class Subnets(AwsAsset):
    _client_name = 'ec2'
    _des_request = 'describe_subnets'
    _response_field = 'Subnets'

    _table_name = 'aws_subnets'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in subnet_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'subnet_id', name='aws_uc_subnets'),)
    _field_document = subnet_field_document


@cloud_providers.aws.register('security_groups')
class SecurityGroups(AwsAsset):
    _client_name = 'ec2'
    _des_request = 'describe_security_groups'
    _response_field = 'SecurityGroups'

    _table_name = 'aws_security_groups'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in sg_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'group_id', name='aws_uc_sg'),)
    _field_document = sg_field_document


@cloud_providers.aws.register('network_interfaces')
class NetworkInterfaces(AwsAsset):
    _client_name = 'ec2'
    _des_request = 'describe_network_interfaces'
    _response_field = 'NetworkInterfaces'

    _table_name = 'aws_network_interfaces'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in nt_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'network_interface_id', name='aws_uc_nt'),)
    _field_document = nt_filed_document


@cloud_providers.aws.register('eips')
class NetworkInterface(AwsAsset):
    _client_name = 'ec2'
    _des_request = 'describe_addresses'
    _response_field = 'Addresses'

    _table_name = 'aws_eips'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in eip_assets_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'public_ip', name='aws_uc_eip'),)
    _field_document = nt_filed_document





# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import copy

from asset.asset_register import cloud_providers
from asset.base import TencentAsset, AssetColumn, UniqueConstraint
from tencentcloud.vpc.v20170312.vpc_client import VpcClient, models

vpc_asset_columns = [
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'VpcName', 'type': 'str'},
    {'name': 'CidrBlock', 'type': 'str'},
    {'name': 'Ipv6CidrBlock', 'type': 'str'},
    {'name': 'IsDefault', 'type': 'str'},
    {'name': 'EnableMulticast', 'type': 'str'},
    {'name': 'CreatedTime', 'type': 'str'},
    {'name': 'EnableDhcp', 'type': 'str'},
    {'name': 'DhcpOptionsId', 'type': 'str'},
    {'name': 'DnsServerSet', 'type': 'dict'},
    {'name': 'DomainName', 'type': 'str'},
    {'name': 'TagSet', 'type': 'dict'},
    {'name': 'AssistantCidrSet', 'type': 'dict'}
]
subnet_asset_columns = [
    {'name': 'NetworkAclId', 'type': 'str'},
    {'name': 'RouteTableId', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'EnableBroadcast', 'type': 'str'},
    {'name': 'Zone', 'type': 'str'},
    {'name': 'Ipv6CidrBlock', 'type': 'str'},
    {'name': 'AvailableIpAddressCount', 'type': 'int'},
    {'name': 'IsRemoteVpcSnat', 'type': 'str'},
    {'name': 'SubnetName', 'type': 'str'},
    {'name': 'TotalIpAddressCount', 'type': 'int'},
    {'name': 'IsCdcSubnet', 'type': 'int'},
    {'name': 'CdcId', 'type': 'str'},
    {'name': 'TagSet', 'type': 'dict'},
    {'name': 'CreatedTime', 'type': 'str'},
    {'name': 'SubnetId', 'type': 'str'},
    {'name': 'CidrBlock', 'type': 'str'},
    {'name': 'IsDefault', 'type': 'str'}
]
route_table_asset_columns = [
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'RouteTableId', 'type': 'str'},
    {'name': 'RouteTableName', 'type': 'str'},
    {'name': 'AssociationSet', 'type': 'dict'},
    {'name': 'RouteSet', 'type': 'dict'},
    {'name': 'Main', 'type': 'str'},
    {'name': 'TagSet', 'type': 'dict'},
    {'name': 'LocalCidrForCcn', 'type': 'dict'},
    {'name': 'CreatedTime', 'type': 'str'}
]
eip_asset_columns = [
    {'name': 'AddressId', 'type': 'str'},
    {'name': 'AddressName', 'type': 'str'},
    {'name': 'AddressIp', 'type': 'str'},
    {'name': 'AddressStatus', 'type': 'str'},
    {'name': 'AddressType', 'type': 'str'},
    {'name': 'InstanceId', 'type': 'str'},
    {'name': 'NetworkInterfaceId', 'type': 'str'},
    {'name': 'PrivateAddressIp', 'type': 'str'},
    {'name': 'IsArrears', 'type': 'str'},
    {'name': 'IsBlocked', 'type': 'str'},
    {'name': 'IsEipDirectConnection', 'type': 'str'},
    {'name': 'EipAlgType', 'type': 'dict'},
    {'name': 'LocalBgp', 'type': 'str'},
    {'name': 'CascadeRelease', 'type': 'str'},
    {'name': 'CreatedTime', 'type': 'str'},
    {'name': 'InternetChargeType', 'type': 'str'},
    {'name': 'Bandwidth', 'type': 'int'},
    {'name': 'InternetServiceProvider', 'type': 'str'},
    {'name': 'TagSet', 'type': 'dict'}
]
network_interface_asset_columns = [
    {'name': 'MacAddress', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'Business', 'type': 'str'},
    {'name': 'Zone', 'type': 'str'},
    {'name': 'NetworkInterfaceId', 'type': 'str'},
    {'name': 'Primary', 'type': 'str'},
    {'name': 'CdcId', 'type': 'str'},
    {'name': 'PrivateIpAddressSet', 'type': 'dict'},
    {'name': 'NetworkInterfaceDescription', 'type': 'str'},
    {'name': 'Ipv6AddressSet', 'type': 'dict'},
    {'name': 'State', 'type': 'str'},
    {'name': 'GroupSet', 'type': 'dict'},
    {'name': 'Attachment', 'type': 'dict'},
    {'name': 'TagSet', 'type': 'dict'},
    {'name': 'EniType', 'type': 'int'},
    {'name': 'CreatedTime', 'type': 'str'},
    {'name': 'SubnetId', 'type': 'str'},
    {'name': 'NetworkInterfaceName', 'type': 'str'},
    {'name': 'AttachType', 'type': 'int'}
]
nat_asset_columns = [
    {'name': 'NatGatewayId', 'type': 'str'},
    {'name': 'NatGatewayName', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'CreatedTime', 'type': 'str'},
    {'name': 'State', 'type': 'str'},
    {'name': 'NetworkState', 'type': 'str'},
    {'name': 'InternetMaxBandwidthOut', 'type': 'int'},
    {'name': 'MaxConcurrentConnection', 'type': 'int'},
    {'name': 'SecurityGroupSet', 'type': 'dict'},
    {'name': 'ExclusiveGatewayBandwidth', 'type': 'int'},
    {'name': 'IsExclusive', 'type': 'str'},
    {'name': 'SubnetId', 'type': 'str'},
    {'name': 'DirectConnectGatewayIds', 'type': 'dict'},
    {'name': 'SourceIpTranslationNatRuleSet', 'type': 'dict'},
    {'name': 'PublicIpAddressSet', 'type': 'dict'},
    {'name': 'DestinationIpPortTranslationNatRuleSet', 'type': 'dict'},
    {'name': 'Zone', 'type': 'str'},
    {'name': 'TagSet', 'type': 'dict'}
]
security_group_asset_columns = [
    {'name': 'SecurityGroupId', 'type': 'str'},
    {'name': 'SecurityGroupName', 'type': 'str'},
    {'name': 'SecurityGroupDesc', 'type': 'str'},
    {'name': 'ProjectId', 'type': 'str'},
    {'name': 'IsDefault', 'type': 'str'},
    {'name': 'CreatedTime', 'type': 'str'},
    {'name': 'Ingress', 'type': 'dict'},
    {'name': 'Egress', 'type': 'dict'}
]

vpc_field_document = 'https://cloud.tencent.com/document/api/215/15778'
subnet_field_document = 'https://cloud.tencent.com/document/api/215/15784'
route_table_field_document = 'https://cloud.tencent.com/document/api/215/15763'
eip_field_document = 'https://cloud.tencent.com/document/api/215/16702'
network_interface_field_document = 'https://cloud.tencent.com/document/api/215/15817'
nat_field_document = 'https://cloud.tencent.com/document/api/215/36034'
security_group_field_document = 'https://cloud.tencent.com/document/api/215/15808'


@cloud_providers.tencent.register('vpcs')
class Vpc(TencentAsset):
    _des_request_func = 'DescribeVpcs'
    _des_request = models.DescribeVpcsRequest()
    _response_field = 'VpcSet'

    _paginate_type = 'str'

    _table_name = 'tencent_vpcs'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in vpc_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'vpc_id', name='tencent_uc_vpc'),)
    _field_document = vpc_field_document

    def _get_client(self):
        return VpcClient(self.cred, self.region)


@cloud_providers.tencent.register('subnets')
class Subnet(TencentAsset):
    _des_request_func = 'DescribeSubnets'
    _des_request = models.DescribeSubnetsRequest()
    _response_field = 'SubnetSet'

    _paginate_type = 'str'

    _table_name = 'tencent_subnets'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in subnet_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'subnet_id', name='tencent_uc_subnet'),)
    _field_document = subnet_field_document

    def _get_client(self):
        return VpcClient(self.cred, self.region)


@cloud_providers.tencent.register('route_tables')
class RouteTable(TencentAsset):
    _des_request_func = 'DescribeRouteTables'
    _des_request = models.DescribeRouteTablesRequest()
    _response_field = 'RouteTableSet'

    _paginate_type = 'str'

    _table_name = 'tencent_route_tables'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in route_table_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'route_table_id', name='tencent_uc_rt'),)
    _field_document = route_table_field_document

    def _get_client(self):
        return VpcClient(self.cred, self.region)


@cloud_providers.tencent.register('eips')
class Eip(TencentAsset):
    _des_request_func = 'DescribeAddresses'
    _des_request = models.DescribeAddressesRequest()
    _response_field = 'AddressSet'

    _table_name = 'tencent_eips'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in eip_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'address_id', name='tencent_uc_eip'),)
    _field_document = eip_field_document

    def _get_client(self):
        return VpcClient(self.cred, self.region)


@cloud_providers.tencent.register('network_interfaces')
class NetworkInterfaces(TencentAsset):
    _des_request_func = 'DescribeNetworkInterfaces'
    _des_request = models.DescribeNetworkInterfacesRequest()
    _response_field = 'NetworkInterfaceSet'

    _table_name = 'tencent_network_interfaces'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in network_interface_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'network_interface_id', name='tencent_uc_nt'),)
    _field_document = network_interface_field_document

    def _get_client(self):
        return VpcClient(self.cred, self.region)


@cloud_providers.tencent.register('nats')
class Nat(TencentAsset):
    _des_request_func = 'DescribeNatGateways'
    _des_request = models.DescribeNetworkInterfacesRequest()
    _response_field = 'NatGatewaySet'

    _table_name = 'tencent_nats'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in nat_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'nat_gateway_id', name='tencent_uc_nat'),)
    _field_document = nat_field_document

    def _get_client(self):
        return VpcClient(self.cred, self.region)


@cloud_providers.tencent.register('security_groups')
class SecurityGroups(TencentAsset):
    _des_request_func = 'DescribeSecurityGroups'
    _des_request = models.DescribeSecurityGroupsRequest()
    _response_field = 'SecurityGroupSet'

    _paginate_type = 'str'

    _table_name = 'tencent_security_groups'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in security_group_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'security_group_id', name='tencent_uc_sg'),)
    _field_document = security_group_field_document

    def _paginate_all_assets(self):
        page, assets = 0, []
        _des_request = copy.deepcopy(self._des_request)
        _des_request.Limit = 50 if self._paginate_type == 'int' else '50'

        while True:
            response = self._describe(
                self.client, self._des_request_func, _des_request, self._response_field).parser_response()
            if not response:
                break
            for security_group in response:
                """get security group rules"""
                security_id = security_group['SecurityGroupId']
                rule_des_request = models.DescribeSecurityGroupPoliciesRequest()
                rule_des_request.SecurityGroupId = security_id

                rule = self._describe(
                    self.client,
                    'DescribeSecurityGroupPolicies',
                    rule_des_request,
                    'SecurityGroupPolicySet'
                ).parser_response()
                security_group['Ingress'] = rule.get('Ingress', [])
                security_group['Egress'] = rule.get('Egress', [])

            assets += response
            page = 1
            if isinstance(_des_request.Limit, str):
                _des_request.Offset = str(page * int(_des_request.Limit))
            else:
                _des_request.Offset = page * _des_request.Limit
        return assets

    def _get_client(self):
        return VpcClient(self.cred, self.region)



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import TencentAsset, AssetColumn, UniqueConstraint
from tencentcloud.clb.v20180317.clb_client import ClbClient, models

asset_columns = [
    {'name': 'LoadBalancerId', 'type': 'str'},
    {'name': 'LoadBalancerName', 'type': 'str'},
    {'name': 'Forward', 'type': 'int'},
    {'name': 'Domain', 'type': 'str'},
    {'name': 'LoadBalancerVips', 'type': 'dict'},
    {'name': 'LoadBalancerType', 'type': 'str'},
    {'name': 'Status', 'type': 'int'},
    {'name': 'CreateTime', 'type': 'str'},
    {'name': 'StatusTime', 'type': 'str'},
    {'name': 'ProjectId', 'type': 'int'},
    {'name': 'OpenBgp', 'type': 'int'},
    {'name': 'Snat', 'type': 'str'},
    {'name': 'Isolation', 'type': 'int'},
    {'name': 'Log', 'type': 'str'},
    {'name': 'AnycastZone', 'type': 'str'},
    {'name': 'AddressIPVersion', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'NumericalVpcId', 'type': 'int'},
    # {'name': 'TargetRegionInfo', 'type': 'dict'},
    {'name': 'SubnetId', 'type': 'str'},
    {'name': 'SecureGroups', 'type': 'dict'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'VipIsp', 'type': 'str'},
    {'name': 'MasterZone', 'type': 'dict'},
    {'name': 'BackupZoneSet', 'type': 'dict'},
    {'name': 'IsolatedTime', 'type': 'str'},
    {'name': 'ExpireTime', 'type': 'str'},
    {'name': 'ChargeType', 'type': 'str'},
    {'name': 'NetworkAttributes', 'type': 'dict'},
    {'name': 'PrepaidAttributes', 'type': 'dict'}
]

clb_field_document = 'https://cloud.tencent.com/document/api/214/30685'


@cloud_providers.tencent.register('clb')
class Clb(TencentAsset):
    _des_request_func = 'DescribeLoadBalancers'
    _des_request = models.DescribeLoadBalancersRequest()
    _response_field = 'LoadBalancerSet'

    _table_name = 'tencent_clb'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'load_balancer_id', name='tencent_uc_clb'),)
    _field_document = clb_field_document

    def _get_client(self):
        return ClbClient(self.cred, self.region)



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import AliyunAsset, AssetColumn, UniqueConstraint
from aliyunsdkcore.client import AcsClient
from aliyunsdkecs.request.v20140526.DescribeInstancesRequest import DescribeInstancesRequest
from aliyunsdkecs.request.v20140526.DescribeEipAddressesRequest import DescribeEipAddressesRequest
from aliyunsdkecs.request.v20140526.DescribeNatGatewaysRequest import DescribeNatGatewaysRequest
from aliyunsdkecs.request.v20140526.DescribeNetworkInterfacesRequest import DescribeNetworkInterfacesRequest
from aliyunsdkecs.request.v20140526.DescribeDisksRequest import DescribeDisksRequest
from aliyunsdkecs.request.v20140526.DescribeSecurityGroupsRequest import DescribeSecurityGroupsRequest
from aliyunsdkecs.request.v20140526.DescribeSecurityGroupAttributeRequest import DescribeSecurityGroupAttributeRequest


asset_columns = [
    {'name': 'CreationTime', 'type': 'str', 'len': 64},
    {'name': 'SerialNumber', 'type': 'str', 'len': 128},
    {'name': 'Status', 'type': 'str', 'len': 16},
    {'name': 'DeploymentSetId', 'type': 'str', 'len': 128},
    {'name': 'KeyPairName', 'type': 'str', 'len': 128},
    {'name': 'SpotStrategy', 'type': 'str', 'len': 32},
    {'name': 'DeviceAvailable', 'type': 'bool'},
    {'name': 'LocalStorageCapacity', 'type': 'float'},
    {'name': 'Description', 'type': 'str', 'len': 256},
    {'name': 'SpotDuration', 'type': 'int'},
    {'name': 'InstanceNetworkType', 'type': 'str', 'len': 16},
    {'name': 'InstanceName', 'type': 'str', 'len': 64},
    {'name': 'OSNameEn', 'type': 'str', 'len': 32},
    {'name': 'HpcClusterId', 'type': 'str', 'len': 128},
    {'name': 'SpotPriceLimit', 'type': 'float'},
    {'name': 'Memory', 'type': 'int', 'len': 128},
    {'name': 'OSName', 'type': 'str', 'len': 32},
    {'name': 'DeploymentSetGroupNo', 'type': 'int'},
    {'name': 'ImageId', 'type': 'str', 'len': 128},
    {'name': 'GPUSpec', 'type': 'str', 'len': 64},
    {'name': 'AutoReleaseTime', 'type': 'str', 'len': 64},
    {'name': 'DeletionProtection', 'type': 'bool'},
    {'name': 'StoppedMode', 'type': 'str', 'len': 16},
    {'name': 'GPUAmount', 'type': 'int'},
    {'name': 'HostName', 'type': 'str', 'len': 128},
    {'name': 'InstanceId', 'type': 'str', 'len': 128},
    {'name': 'InternetMaxBandwidthOut', 'type': 'int'},
    {'name': 'InternetMaxBandwidthIn', 'type': 'int'},
    {'name': 'InstanceType', 'type': 'str', 'len': 64},
    {'name': 'InstanceChargeType', 'type': 'str', 'len': 32},
    {'name': 'RegionId', 'type': 'str', 'len': 16},
    {'name': 'IoOptimized', 'type': 'bool'},
    {'name': 'StartTime', 'type': 'str', 'len': 64},
    {'name': 'Cpu', 'type': 'int'},
    {'name': 'LocalStorageAmount', 'type': 'int'},
    {'name': 'ExpiredTime', 'type': 'str', 'len': 64},
    {'name': 'ResourceGroupId', 'type': 'str', 'len': 128},
    {'name': 'InternetChargeType', 'type': 'str', 'len': 32},
    {'name': 'ZoneId', 'type': 'str', 'len': 32},
    {'name': 'Recyclable', 'type': 'bool'},
    {'name': 'CreditSpecification', 'type': 'str', 'len': 16},
    {'name': 'InstanceTypeFamily', 'type': 'str', 'len': 16},
    {'name': 'OSType', 'type': 'str', 'len': 8},
    {'name': 'NetworkInterfaces', 'type': 'dict'},
    {'name': 'OperationLocks', 'type': 'dict'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'RdmaIpAddress', 'type': 'dict'},
    {'name': 'SecurityGroupIds', 'type': 'dict'},
    {'name': 'PublicIpAddress', 'type': 'dict'},
    {'name': 'InnerIpAddress', 'type': 'dict'},
    {'name': 'VpcAttributes', 'type': 'dict'},
    {'name': 'EipAddress', 'type': 'dict'},
    {'name': 'HibernationOptions', 'type': 'dict'},
    {'name': 'DedicatedHostAttribute', 'type': 'dict'},
    {'name': 'EcsCapacityReservationAttr', 'type': 'dict'},
    {'name': 'DedicatedInstanceAttribute', 'type': 'dict'},
    {'name': 'CpuOptions', 'type': 'dict'},
    {'name': 'MetadataOptions', 'type': 'dict'},
    {'name': 'ImageOptions', 'type': 'dict'}
]
eip_asset_columns = [
    {'name': 'ReservationActiveTime', 'type': 'str'},
    {'name': 'Status', 'type': 'str'},
    {'name': 'ReservationOrderType', 'type': 'str'},
    {'name': 'AllocationTime', 'type': 'str'},
    {'name': 'Netmode', 'type': 'str'},
    {'name': 'ChargeType', 'type': 'str'},
    {'name': 'Description', 'type': 'str'},
    {'name': 'SegmentInstanceId', 'type': 'str'},
    {'name': 'ReservationInternetChargeType', 'type': 'str'},
    {'name': 'BandwidthPackageId', 'type': 'str'},
    {'name': 'IpAddress', 'type': 'str'},
    {'name': 'Bandwidth', 'type': 'str'},
    {'name': 'ReservationBandwidth', 'type': 'str'},
    {'name': 'EipBandwidth', 'type': 'str'},
    {'name': 'Name', 'type': 'str'},
    {'name': 'InstanceRegionId', 'type': 'str'},
    {'name': 'DeletionProtection', 'type': 'str'},
    {'name': 'InstanceId', 'type': 'str'},
    {'name': 'SecondLimited', 'type': 'str'},
    {'name': 'InstanceType', 'type': 'str'},
    {'name': 'HDMonitorStatus', 'type': 'str'},
    {'name': 'RegionId', 'type': 'str'},
    {'name': 'BandwidthPackageBandwidth', 'type': 'str'},
    {'name': 'ServiceManaged', 'type': 'int'},
    {'name': 'ExpiredTime', 'type': 'str'},
    {'name': 'ResourceGroupId', 'type': 'str'},
    {'name': 'AllocationId', 'type': 'str'},
    {'name': 'InternetChargeType', 'type': 'str'},
    {'name': 'BusinessStatus', 'type': 'str'},
    {'name': 'BandwidthPackageType', 'type': 'str'},
    {'name': 'HasReservationData', 'type': 'str'},
    {'name': 'ISP', 'type': 'str'},
    {'name': 'OperationLocks', 'type': 'dict'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'SecurityProtectionTypes', 'type': 'dict'},
    {'name': 'PublicIpAddressPoolId', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'Zone', 'type': 'str'}
]
nat_asset_columns = [
    {'name': 'Status', 'type': 'str'},
    {'name': 'CreationTime', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'Spec', 'type': 'str'},
    {'name': 'Description', 'type': 'str'},
    {'name': 'NatGatewayId', 'type': 'str'},
    {'name': 'BusinessStatus', 'type': 'str'},
    {'name': 'Name', 'type': 'str'},
    {'name': 'InstanceChargeType', 'type': 'str'},
    {'name': 'RegionId', 'type': 'str'},
    {'name': 'ForwardTableIds', 'type': 'dict'},
    {'name': 'BandwidthPackageIds', 'type': 'dict'}
]
network_interface_columns = [
    {'name': 'CreationTime', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'Type', 'type': 'str'},
    {'name': 'Status', 'type': 'str'},
    {'name': 'NetworkInterfaceTrafficMode', 'type': 'str'},
    {'name': 'NetworkInterfaceName', 'type': 'str'},
    {'name': 'MacAddress', 'type': 'str'},
    {'name': 'QueuePairNumber', 'type': 'int'},
    {'name': 'NetworkInterfaceId', 'type': 'str'},
    {'name': 'ServiceID', 'type': 'str'},
    {'name': 'InstanceId', 'type': 'str'},
    {'name': 'OwnerId', 'type': 'str'},
    {'name': 'ServiceManaged', 'type': 'str'},
    {'name': 'VSwitchId', 'type': 'str'},
    {'name': 'Description', 'type': 'str'},
    {'name': 'ResourceGroupId', 'type': 'str'},
    {'name': 'ZoneId', 'type': 'str'},
    {'name': 'PrivateIpAddress', 'type': 'str'},
    {'name': 'QueueNumber', 'type': 'int'},
    {'name': 'PrivateIpSets', 'type': 'dict'},
    {'name': 'Ipv6Sets', 'type': 'dict'},
    {'name': 'Ipv4PrefixSets', 'type': 'dict'},
    {'name': 'Ipv6PrefixSets', 'type': 'dict'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'SecurityGroupIds', 'type': 'dict'},
    {'name': 'AssociatedPublicIp', 'type': 'dict'},
    {'name': 'Attachment', 'type': 'dict'}
]
disk_interface_columns = [
    {'name': 'SerialNumber', 'type': 'str'},
    {'name': 'CreationTime', 'type': 'str'},
    {'name': 'Status', 'type': 'str'},
    {'name': 'Type', 'type': 'str'},
    {'name': 'PerformanceLevel', 'type': 'str'},
    {'name': 'BdfId', 'type': 'str'},
    {'name': 'EnableAutoSnapshot', 'type': 'str'},
    {'name': 'StorageSetId', 'type': 'str'},
    {'name': 'StorageSetPartitionNumber', 'type': 'int'},
    {'name': 'DiskId', 'type': 'str'},
    {'name': 'DeleteAutoSnapshot', 'type': 'str'},
    {'name': 'StorageClusterId', 'type': 'str'},
    {'name': 'Encrypted', 'type': 'str'},
    {'name': 'IOPSRead', 'type': 'int'},
    {'name': 'MountInstanceNum', 'type': 'int'},
    {'name': 'Description', 'type': 'str'},
    {'name': 'Device', 'type': 'str'},
    {'name': 'DiskName', 'type': 'str'},
    {'name': 'Portable', 'type': 'str'},
    {'name': 'ImageId', 'type': 'str'},
    {'name': 'KMSKeyId', 'type': 'str'},
    {'name': 'DeleteWithInstance', 'type': 'str'},
    {'name': 'DetachedTime', 'type': 'str'},
    {'name': 'SourceSnapshotId', 'type': 'str'},
    {'name': 'AutoSnapshotPolicyId', 'type': 'str'},
    {'name': 'EnableAutomatedSnapshotPolicy', 'type': 'str'},
    {'name': 'IOPSWrite', 'type': 'int'},
    {'name': 'InstanceId', 'type': 'str'},
    {'name': 'IOPS', 'type': 'int'},
    {'name': 'RegionId', 'type': 'str'},
    {'name': 'ExpiredTime', 'type': 'str'},
    {'name': 'Size', 'type': 'int'},
    {'name': 'ResourceGroupId', 'type': 'str'},
    {'name': 'DiskChargeType', 'type': 'str'},
    {'name': 'ZoneId', 'type': 'str'},
    {'name': 'AttachedTime', 'type': 'str'},
    {'name': 'Category', 'type': 'str'},
    {'name': 'ProductCode', 'type': 'str'},
    {'name': 'MultiAttach', 'type': 'str'},
    {'name': 'OperationLocks', 'type': 'dict'},
    {'name': 'MountInstances', 'type': 'dict'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'Attachments', 'type': 'dict'},
    {'name': 'ProvisionedIops', 'type': 'int'},
    {'name': 'BurstingEnabled', 'type': 'str'},
    {'name': 'Throughput', 'type': 'int'}
]
nat_interface_columns = [
    {'name': 'id', 'type': 'str'},
    {'name': 'router_id', 'type': 'str'},
    {'name': 'status', 'type': 'str'},
    {'name': 'description', 'type': 'str'},
    {'name': 'admin_state_up', 'type': 'str'},
    {'name': 'tenant_id', 'type': 'str'},
    {'name': 'created_at', 'type': 'str'},
    {'name': 'spec', 'type': 'str'},
    {'name': 'internal_network_id', 'type': 'str'},
    {'name': 'name', 'type': 'str'},
    {'name': 'enterprise_project_id', 'type': 'str'}
]
sg_interface_columns = [
    {'name': 'SecurityGroupId', 'type': 'str'},
    {'name': 'SecurityGroupName', 'type': 'str'},
    {'name': 'Description', 'type': 'str'},
    {'name': 'SecurityGroupType', 'type': 'str'},
    {'name': 'VpcId', 'type': 'str'},
    {'name': 'CreationTime', 'type': 'str'},
    {'name': 'EcsCount', 'type': 'int'},
    {'name': 'AvailableInstanceAmount', 'type': 'int'},
    {'name': 'ResourceGroupId', 'type': 'str'},
    {'name': 'ServiceManaged', 'type': 'str'},
    {'name': 'ServiceID', 'type': 'str'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'Permissions', 'type': 'dict'}
]

ecs_field_document = 'https://help.aliyun.com/document_detail/25506.html?spm=a2c4g.11186623.0.0.3ac56cf0qyfOQ7#resultMapping'
eip_field_document = 'https://next.api.aliyun.com/document/Ecs/2014-05-26/DescribeEipAddresses#workbench-doc-params'
nat_field_document = 'https://next.api.aliyun.com/document/Ecs/2014-05-26/DescribeNatGateways#workbench-doc-response'
network_interface_field_document = 'https://next.api.aliyun.com/document/Ecs/2014-05-26/DescribeNetworkInterfaces'
disk_field_document = 'https://next.api.aliyun.com/document/Ecs/2014-05-26/DescribeDisks'
sg_field_document = 'https://next.api.aliyun.com/document/Ecs/2014-05-26/DescribeSecurityGroups'


@cloud_providers.aliyun.register('ecs')
class Ecs(AliyunAsset):
    _des_request = DescribeInstancesRequest
    _response_field = 'Instances'
    _child_response_filed = 'Instance'

    _table_name = 'aliyun_ecs'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'instance_id', name='aliyun_instance'),)
    _field_document = ecs_field_document

    def _get_client(self):
        return AcsClient(credential=self.cred, region_id=self.region)


@cloud_providers.aliyun.register('eip')
class Eip(Ecs):
    _des_request = DescribeEipAddressesRequest
    _response_field = 'EipAddresses'
    _child_response_filed = 'EipAddress'

    _table_name = 'aliyun_eips'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in eip_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'ip_address', name='aliyun_uc_eip'),)
    _field_document = eip_field_document


@cloud_providers.aliyun.register('nat_gateway')
class NatGateway(Ecs):
    _des_request = DescribeNatGatewaysRequest
    _response_field = 'NatGateways'
    _child_response_filed = 'NatGateway'

    _table_name = 'aliyun_nat_gateway'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in nat_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'nat_gateway_id', name='aliyun_uc_ng'),)
    _field_document = eip_field_document


@cloud_providers.aliyun.register('network_interface')
class NetworkInterface(Ecs):
    _des_request = DescribeNetworkInterfacesRequest
    _response_field = 'NetworkInterfaceSets'
    _child_response_filed = 'NetworkInterfaceSet'

    _table_name = 'aliyun_network_interface'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in network_interface_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'network_interface_id', name='aliyun_uc_nii'),)
    _field_document = network_interface_field_document


@cloud_providers.aliyun.register('disk')
class Disk(Ecs):
    _des_request = DescribeDisksRequest
    _response_field = 'Disks'
    _child_response_filed = 'Disk'

    _table_name = 'aliyun_disk'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in disk_interface_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'disk_id', name='aliyun_uc_disk'),)
    _field_document = disk_field_document


@cloud_providers.aliyun.register('security_groups')
class Sg(Ecs):
    _des_request = DescribeSecurityGroupsRequest
    _response_field = 'SecurityGroups'
    _child_response_filed = 'SecurityGroup'

    _table_name = 'aliyun_sg'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in sg_interface_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'security_group_id', name='aliyun_uc_sg'),)
    _field_document = sg_field_document

    def _paginate_all_assets(self):
        assets = super(Sg, self)._paginate_all_assets()
        for asset in assets:
            sg_id = asset['SecurityGroupId']
            request = DescribeSecurityGroupAttributeRequest()
            request.set_query_params({'SecurityGroupId': sg_id, 'RegionId': self.region})
            response = self._describe(
                self.client,
                request,
                'Permissions',
                'Permission',
                self.parser_response
            ).parser_response()
            if not response:
                continue
            asset['Permissions'] = response

        return assets




# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import os
import arrow
from sqlalchemy import Integer, String, JSON, Date, Text, DateTime, NUMERIC, DECIMAL, FLOAT
from asset.schema import TencentProfile, AliyunProfile, AwsProfile, HuaweiProfile


PROVIDERS = ('tencent', 'aliyun', 'aws', 'huawei')
PROVIDER_PROFILES = {
    'tencent': TencentProfile,
    'aliyun': AliyunProfile,
    'aws': AwsProfile,
    'huawei': HuaweiProfile
}

today = arrow.now().datetime
yesterday = arrow.now().shift(days=-1).datetime

CLOUD_ASSERT_DB = {
    'host': os.getenv('CLOUD_ASSERT_HOST', ''),
    'port': os.getenv('CLOUD_ASSERT_PORT', ''),
    'password': os.getenv('CLOUD_ASSERT_PASSWORD', ''),
    'user': os.getenv('CLOUD_ASSERT_USER', ''),
    'database': os.getenv('CLOUD_ASSERT_DATABASE', ''),
}

COLUMNS_MAP = {
    'str': String,
    'int': Integer,
    'dict': JSON,
    'list': JSON,
    'date': Date,
    'datetime': DateTime,
    'text': Text,
    'bool': String,
    'numeric': NUMERIC,
    'decimal': DECIMAL,
    'float': FLOAT
}

# todo: complete cloud providers`s regions
REGIONS = {
    'tencent': {
        'default_region': 'ap-shanghai',
        'regions': [
            'ap-beijing',
            'ap-guangzhou',
            'ap-hongkong',
            'ap-shanghai',
            'ap-shanghai-fsi',
            'ap-shenzhen-fsi',
            'ap-nanjing',
            'ap-chengdu',
            'ap-chongqing'
        ]
    },
    'aliyun': {
        'default_region': 'cn-beijing',
        'regions': [
            'cn-qingdao',
            'cn-beijing',
            'cn-chengdu',
            'cn-zhangjiakou',
            'cn-huhehaote',
            'cn-wulanchabu',
            'cn-hangzhou',
            'cn-shanghai',
            'cn-nanjing',
            'cn-fuzhou',
            'cn-shenzhen',
            'cn-heyuan',
            'cn-guangzhou',
            'cn-hongkong',

            'ap-southeast-1',
            'ap-southeast-2',
            'ap-southeast-3',
            'ap-southeast-5',
            'ap-southeast-6',
            'ap-southeast-7',
            'ap-south-1',
            'ap-northeast-1',
            'ap-northeast-2',
            'us-west-1',
            'us-east-1',
            'eu-central-1',
            'eu-west-1',
            'me-east-1'
        ]
    },
    'aws': {
        'default_region': 'cn-northwest-1',
        'regions': [
            'cn-northwest-1',
            'cn-north-1'
        ]
    },
    'huawei': {
        'default_region': 'cn-south-1',
        'regions': [
            'cn-north-1',
            'cn-north-4',
            'cn-south-1',
            'cn-east-2',
            'cn-east-3',
            'cn-southwest-2',
            'ap-southeast-1',
            'ap-southeast-2',
            'ap-southeast-3',
            'af-south-1',
            'sa-brazil-1',
            'la-north-2',
            'cn-south-4',
            'na-mexico-1',
            'la-south-2',
            'cn-south-2',
            'cn-north-9',
            'cn-north-2',
            'ap-southeast-4'
        ]
    }
}



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import copy
import datetime
from asset.base import AwsAsset, AssetColumn, UniqueConstraint
from asset.asset_register import cloud_providers
from asset.conf import today, yesterday

ecu_asset_columns = [
    {'name': 'InstanceId', 'type': 'str', 'len': 64},
    {'name': 'Metric', 'type': 'str', 'len': 16},
    {'name': 'MetricTimestamp', 'type': 'datetime'},
    {'name': 'MetricValue', 'type': 'float'},
    {'name': 'Period', 'type': 'int'}
]
period = 3600
ecu_metric_dq = [
    {
        'Id': 'm{instance_id}1',
        'Label': 'Minimum::{instance_id}',
        'MetricStat': {
            'Metric': {
                'Namespace': 'AWS/EC2',
                'MetricName': 'CPUUtilization',
                'Dimensions': []
            },
            'Period': period,
            'Stat': 'Minimum'
        }
    },
    {
        'Id': 'm{instance_id}2',
        'Label': 'Maximum::{instance_id}',
        'MetricStat': {
            'Metric': {
                'Namespace': 'AWS/EC2',
                'MetricName': 'CPUUtilization',
                'Dimensions': []
            },
            'Period': period,
            'Stat': 'Maximum'
        }
    }
]
ecu_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudwatch.html#CloudWatch.Client.get_metric_data'


@cloud_providers.aws.register('ec2_cpu_utilization')
class Ec2CpuUtilization(AwsAsset):
    """
        get ec2_cpu_utilization metric, include (Minimum, Maximum)
    """
    _client_name = 'cloudwatch'
    _des_request = 'get_metric_data'
    _response_field = 'MetricDataResults'
    _des_request_kwargs: dict = {
        'MetricDataQueries': [],
        'StartTime': datetime.datetime(yesterday.year, yesterday.month, yesterday.day),
        'EndTime': datetime.datetime(today.year, today.month, today.day)
    }
    _field_document = ecu_field_document

    _table_name = 'aws_ec2_cpu_utilization'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in ecu_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'instance_id', 'metric', 'metric_timestamp', name='aws_uc_ec2_cpu_u'),)

    def _paginate_all_assets(self) -> list:
        from asset.aws.ec2 import Ec2
        assets = []
        instances = Ec2(self.cred, region=self.region, dbconfig=self.dbconfig).paginate_all_assets
        pagesize = 50

        for index in range(len(instances) // pagesize + 1):
            _instances = instances[index * pagesize: index * pagesize + pagesize]
            if not _instances:
                continue
            metric_data_queries = []
            for instance in _instances:
                instance_id = instance['InstanceId']
                instance_id_sp = instance_id.split('-')[1]
                dimension = [{'Name': 'InstanceId', 'Value': instance_id}]
                _ecu_metric_dq = copy.deepcopy(ecu_metric_dq)

                # metric: Minimum
                _ecu_metric_dq[0]['MetricStat']['Metric']['Dimensions'] = dimension
                _ecu_metric_dq[0]['Id'] = _ecu_metric_dq[0]['Id'].format(instance_id=instance_id_sp)
                _ecu_metric_dq[0]['Label'] = _ecu_metric_dq[0]['Label'].format(instance_id=instance_id_sp)

                # metric: Maximum
                _ecu_metric_dq[1]['MetricStat']['Metric']['Dimensions'] = dimension
                _ecu_metric_dq[1]['Id'] = _ecu_metric_dq[1]['Id'].format(instance_id=instance_id_sp)
                _ecu_metric_dq[1]['Label'] = _ecu_metric_dq[1]['Label'].format(instance_id=instance_id_sp)

                metric_data_queries += _ecu_metric_dq

            self._des_request_kwargs['MetricDataQueries'] = metric_data_queries
            result = self._describe(
                self.client,
                self._des_request,
                self._response_field,
                self._child_response_filed,
                des_request_kwargs=self._des_request_kwargs,
                parser_response_func=self.parser_response
            ).describe()
            metric_drs = result.get('MetricDataResults', [])
            for metric_dr in metric_drs:
                metric, instance_id = metric_dr['Label'].split('::')
                values = zip(metric_dr['Timestamps'], metric_dr['Values'])
                assets += [
                    {
                        'InstanceId': instance_id,
                        'Metric': metric,
                        'MetricTimestamp': value[0],
                        'MetricValue': value[1],
                        'Period': period
                    } for value in values
                ]
        return sorted(assets, key=lambda a: a['MetricTimestamp'])





# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import AwsAsset, AssetColumn, UniqueConstraint


buckets_filed_document = """
    1、https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.list_buckets
    2、https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.get_bucket_acl
    3、https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.get_bucket_policy
    4、https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.get_bucket_policy_status
"""

buckets_asset_columns = [
    {'name': 'Name', 'type': 'str'},
    {'name': 'CreationDate', 'type': 'datetime'},
    {'name': 'BucketAclGrants', 'type': 'dict'},
    {'name': 'BucketPolicy', 'type': 'text'},
    {'name': 'BucketPolicyStatus', 'type': 'dict'},
]


@cloud_providers.aws.register('s3_buckets')
class S3Buckets(AwsAsset):
    _client_name = 's3'

    _table_name = 'aws_s3_buckets'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in buckets_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'name', name='aws_uc_s3b'),)
    _field_document = buckets_filed_document

    def _paginate_all_assets(self) -> list:
        assets = []
        buckets, _ = self._describe(
            self.client, 'list_buckets', 'Buckets', parser_response_func=self.parser_response).parser_response()
        for bucket in buckets:
            """get_bucket_acl"""
            des_request_kwargs = {'Bucket': bucket['Name']}
            try:
                bucket_acl_grants, _ = self._describe(
                    self.client, 'get_bucket_acl', 'Grants',
                    des_request_kwargs=des_request_kwargs, parser_response_func=self.parser_response
                ).parser_response()
            except Exception as error:
                bucket_acl_grants = [str(error)]

            try:
                bucket_policy, _ = self._describe(
                    self.client, 'get_bucket_policy', 'Policy',
                    des_request_kwargs=des_request_kwargs, parser_response_func=self.parser_response
                ).parser_response()
            except Exception as error:
                bucket_policy = str(error)

            try:
                bucket_policy_status, _ = self._describe(
                    self.client, 'get_bucket_policy_status', 'PolicyStatus',
                    des_request_kwargs=des_request_kwargs, parser_response_func=self.parser_response
                ).parser_response()
            except Exception as error:
                bucket_policy_status = {'error': str(error)}

            bucket.update(
                {
                    'BucketAclGrants': bucket_acl_grants,
                    'BucketPolicy': bucket_policy,
                    'BucketPolicyStatus': bucket_policy_status
                }
            )
            assets.append(bucket)
        return assets




# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import os
import click

from yaml import load, Loader

from asset.commands import Fetch
from asset.schema import DbConfig
from asset.utils import register_assets
from asset.conf import PROVIDERS, REGIONS, PROVIDER_PROFILES
from asset.exceptions import BaseCloudAssetException


@click.group()
def main():
    pass


@click.command()
@click.option('--cloud-provider', help=f'support cloud providers {PROVIDERS}', type=str)
@click.option('--profile-path', help='', type=str)
@click.option('--assets', help='', type=str)
@click.option('--regions', help='', type=str)
@click.option('--log-dir-path', help='', type=str, default='./')
def fetch(cloud_provider, profile_path, assets, regions=None, log_dir_path='./'):
    if cloud_provider not in PROVIDERS:
        raise BaseCloudAssetException(f'not support {cloud_provider}, only support cloud providers {PROVIDERS}')

    if not os.path.exists(profile_path):
        raise BaseCloudAssetException(f'not find the profile`s file by {profile_path}')

    if assets is None:
        raise BaseCloudAssetException('assets not be None!')

    with open(profile_path, 'r') as f:
        profile = load(f, Loader=Loader)

    dbconfig = profile.get('dbconfig', None)
    if dbconfig is None:
        dbconfig = DbConfig()
    else:
        dbconfig = DbConfig(**profile.pop('dbconfig'))

    cloud_providers = profile.get('cloud_providers', None)
    if cloud_providers is None:
        raise BaseCloudAssetException('cloud_providers not be None')
    cloud_provider_profile = filter(lambda x: x['cloud_provider'] == cloud_provider, cloud_providers).__next__()
    cloud_provider_profile.setdefault('regions', None)
    _regions = cloud_provider_profile.pop('regions')
    if regions is None:
        regions = _regions if _regions else REGIONS[cloud_provider]['default_region'].split(',')
    else:
        cloud_regions = REGIONS[cloud_provider]['regions']
        if regions == 'all':
            regions = cloud_regions
        elif regions == 'none':
            regions = cloud_regions[0: 1]
        else:
            regions = set(regions.split(','))
            diff_regions = regions.difference(cloud_regions)
            if diff_regions:
                raise BaseCloudAssetException(f'not support regions: {regions}, just support: {cloud_regions}')
            regions = list(regions)

    if assets == 'all':
        assets = 'all'
    else:
        assets = set(assets.split(','))
        if not assets:
            raise BaseCloudAssetException(f'not find assets: {assets}')
        diff_assets = assets.difference(set(list(register_assets(cloud_provider=cloud_provider).assets.keys())))
        if diff_assets:
            raise BaseCloudAssetException(f'not find {cloud_provider}`assets: {diff_assets}')
        assets = list(assets)
    cloud_provider_profile = PROVIDER_PROFILES[cloud_provider](**cloud_provider_profile)

    Fetch(cloud_provider, cloud_provider_profile, assets, regions, log_dir_path=log_dir_path, dbconfig=dbconfig).fetch()


main.add_command(fetch)


if __name__ == '__main__':
    main()



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0


from asset.asset_register import cloud_providers
from asset.base import TencentAsset, AssetColumn, UniqueConstraint
from tencentcloud.cam.v20190116.cam_client import CamClient, models


user_asset_columns = [
    {'name': 'Uin', 'type': 'str'},
    {'name': 'Name', 'type': 'str'},
    {'name': 'Uid', 'type': 'str'},
    {'name': 'Remark', 'type': 'str'},
    {'name': 'ConsoleLogin', 'type': 'str'},
    {'name': 'PhoneNum', 'type': 'str'},
    {'name': 'CountryCode', 'type': 'str'},
    {'name': 'Email', 'type': 'str'},
    {'name': 'CreateTime', 'type': 'str'}
]
user_ak_asset_columns = [
    {'name': 'AccessKeyId', 'type': 'str'},
    {'name': 'Status', 'type': 'str'},
    {'name': 'CreateTime', 'type': 'str'},
    {'name': 'LastUse', 'type': 'dict'},
]
role_columns = [
    {'name': 'AddTime', 'type': 'str'},
    {'name': 'ConsoleLogin', 'type': 'int'},
    {'name': 'DeletionTaskId', 'type': 'str'},
    {'name': 'Description', 'type': 'str'},
    {'name': 'PolicyDocument', 'type': 'str'},
    {'name': 'RoleId', 'type': 'str'},
    {'name': 'RoleName', 'type': 'str'},
    {'name': 'RoleType', 'type': 'str'},
    {'name': 'SessionDuration', 'type': 'int'},
    {'name': 'Tags', 'type': 'dict'},
    {'name': 'UpdateTime', 'type': 'str'}
]
user_asset_document = 'https://cloud.tencent.com/document/api/598/34587'
user_ak_asset_document = 'https://cloud.tencent.com/document/api/598/45156'
role_asset_document = 'https://cloud.tencent.com/document/api/598/36223'



@cloud_providers.tencent.register('user')
class User(TencentAsset):
    _des_request_func = 'ListUsers'
    _des_request = models.ListUsersRequest()
    _response_field = 'Data'

    _paginate = False

    _table_name = 'tencent_user'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in user_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'uid', name='tencent_uc_user'),)
    _field_document = user_asset_document

    def _get_client(self):
        return CamClient(self.cred, self.region)


@cloud_providers.tencent.register('user_ak')
class UserAk(User):
    _des_request_func = 'ListAccessKeys'
    _des_request = models.ListAccessKeysRequest()
    _response_field = 'AccessKeys'

    _paginate = False

    _table_name = 'tencent_user_ak'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in user_ak_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'access_key_id', name='tencent_uc_user_ak'),)
    _field_document = user_ak_asset_document

    def _paginate_all_assets(self):
        assets = []
        users = User(self.cred, region=self.region, dbconfig=self.dbconfig)._paginate_all_assets()
        for user in users:
            self._des_request.TargetUin = user['Uin']
            response = self._describe(
                self.client, self._des_request_func, self._des_request, self._response_field).parser_response()
            if not response:
                break

            assets += response
        if not assets:
            return assets

        # get ak details
        request = models.GetSecurityLastUsedRequest()
        request.SecretIdList = [asset['AccessKeyId'] for asset in assets]
        response = self._describe(
            self.client,
            'GetSecurityLastUsed',
            request,
            'SecretIdLastUsedRows'
        ).parser_response()
        for asset in assets:
            asset['LastUse'] = list(filter(lambda x: x['SecretId'] == asset['AccessKeyId'], response))

        return assets


# @cloud_providers.tencent.register('role')
# class Role(TencentAsset):
#     pass





# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0
import copy

from typing import List

from sqlalchemy.dialects.postgresql import insert
from sqlalchemy import MetaData, Table, Column

from asset.conf import COLUMNS_MAP
from asset.schema import AssetColumn
from asset.utils import to_hump_underline


class AssetTable:

    def __init__(
            self,
            table_name: str,
            engine: object,
            asset_columns: List[AssetColumn],
            default_columns: List[AssetColumn] = None,
            is_hump_underline: bool = True,
            args: tuple = None,
            kwargs: dict = None
    ):
        """
        :param table_name:
        :param engine:
        :param asset_columns:
        :param default_columns:
        :param is_hump_underline:
        :param args: 创建Table类的*args
        :param kwargs: 创建Table类的**kwargs
        """
        self.asset_columns = self.column_to_hump_underline(asset_columns) if is_hump_underline else asset_columns
        self.asset_columns = list() if default_columns is None else default_columns + self.asset_columns

        self.table_name = table_name
        self.engine = engine

        self.args = tuple() if args is None else args
        self.kwargs = dict() if kwargs is None else kwargs

    @property
    def columns(self):
        return self.generate_columns(self.asset_columns)

    @property
    def table(self) -> Table:
        return self.__create_table()

    def __create_table(self) -> Table:
        table = list(filter(lambda table_name: table_name == self.table_name, self.engine.table_names()))
        if table:
            return Table(self.table_name, MetaData(bind=self.engine), *self.columns, *self.args, **self.kwargs)

        table = Table(self.table_name, MetaData(bind=self.engine), *self.columns, *self.args, **self.kwargs)
        try:
            table.create()
        except Exception as e:
            raise e
        return table

    @classmethod
    def generate_columns(cls, asset_columns: List[AssetColumn]) -> list:
        """把字段转化为下sqlalchemy的columns"""
        columns = []
        for column in asset_columns:
            column_type = COLUMNS_MAP[column.type]
            if column.type == 'str':
                columns.append(
                    Column(column.name, column_type(column.length), **column.kwargs))
            elif column.type == 'numeric':
                columns.append(
                    Column(column.name, column_type(20, 10), **column.kwargs))
            elif column.type == 'decimal':
                columns.append(
                    Column(column.name, column_type(20, 10), **column.kwargs))
            else:
                columns.append(Column(column.name, column_type, **column.kwargs))

        return columns

    @classmethod
    def insert_values(cls, table: Table, values: list, engine):
        with engine.connect() as conn:
            conn.execute(insert(table).values(values))

    @classmethod
    def insert_values_duplicat_do_nothing(cls, table, values, engine, constraint):
        with engine.connect() as conn:
            insert_stmt = insert(table).values(values)
            conn.execute(insert_stmt.on_conflict_do_nothing(constraint=constraint))

    @classmethod
    def column_to_hump_underline(cls, columns: List[AssetColumn]) -> List[AssetColumn]:
        _columns = copy.deepcopy(columns)
        for column in _columns:
            column.name = to_hump_underline(column.name)
        return _columns



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import TencentAsset, AssetColumn, UniqueConstraint
from tencentcloud.cdb.v20170320.cdb_client import CdbClient, models

cdb_m_asset_columns = [
    {'name': 'WanStatus', 'type': 'int'},
    {'name': 'Zone', 'type': 'str'},
    {'name': 'WanPort', 'type': 'int'},
    {'name': 'RoVipInfo', 'type': 'dict'},
    {'name': 'Memory', 'type': 'int'},
    {'name': 'EngineType', 'type': 'str'},
    {'name': 'Status', 'type': 'int'},
    {'name': 'VpcId', 'type': 'int'},
    {'name': 'SlaveInfo', 'type': 'dict'},
    {'name': 'InstanceId', 'type': 'str'},
    {'name': 'PhysicalId', 'type': 'str'},
    {'name': 'Volume', 'type': 'int'},
    {'name': 'AutoRenew', 'type': 'int'},
    {'name': 'ProtectMode', 'type': 'int'},
    {'name': 'CdbError', 'type': 'int'},
    {'name': 'DeviceClass', 'type': 'str'},
    {'name': 'MasterInfo', 'type': 'dict'},
    {'name': 'DeployGroupId', 'type': 'str'},
    {'name': 'InstanceNodes', 'type': 'int'},
    {'name': 'RoGroups', 'type': 'dict'},
    {'name': 'ProjectId', 'type': 'int'},
    {'name': 'DeadlineTime', 'type': 'str'},
    {'name': 'DeployMode', 'type': 'int'},
    {'name': 'TaskStatus', 'type': 'int'},
    {'name': 'SubnetId', 'type': 'int'},
    {'name': 'DeviceType', 'type': 'str'},
    {'name': 'EngineVersion', 'type': 'str'},
    {'name': 'MaxDelayTime', 'type': 'int'},
    {'name': 'InstanceName', 'type': 'str'},
    {'name': 'Cpu', 'type': 'int'},
    {'name': 'DrInfo', 'type': 'dict'},
    {'name': 'UniqVpcId', 'type': 'str'},
    {'name': 'WanDomain', 'type': 'str'},
    {'name': 'InitFlag', 'type': 'int'},
    {'name': 'PayType', 'type': 'int'},
    {'name': 'InstanceType', 'type': 'int'},
    {'name': 'Vip', 'type': 'str'},
    {'name': 'UniqSubnetId', 'type': 'str'},
    {'name': 'Qps', 'type': 'int'},
    {'name': 'Vport', 'type': 'int'},
    {'name': 'TagList', 'type': 'dict'},
    {'name': 'CreateTime', 'type': 'str'},
    {'name': 'ZoneId', 'type': 'int'},
    {'name': 'ZoneName', 'type': 'str'}
]

cdb_m_field_document = 'https://cloud.tencent.com/document/api/236/15872#1.-.E6.8E.A5.E5.8F.A3.E6.8F.8F.E8.BF.B0'


@cloud_providers.tencent.register('cdb_mysql')
class CdbMysql(TencentAsset):
    _des_request_func = 'DescribeDBInstances'
    _des_request = models.DescribeDBInstancesRequest()
    _response_field = 'Items'

    _table_name = 'tencent_cdb_mysql'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in cdb_m_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'instance_id', name='tencent_uc_cdb_m'),)
    _field_document = cdb_m_field_document

    def _get_client(self):
        return CdbClient(self.cred, self.region)



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import abc
import copy
import logging
import datetime

from typing import List

from sqlalchemy import create_engine, Table, UniqueConstraint

from boto3 import Session
from aliyunsdkcore.client import AcsClient

from asset.asset_table import AssetTable
from asset.schema import DbConfig, AssetColumn, STSAssumeRoleCredential, RamRoleArnCredential, AwsCredential,\
    TencentProfile, AliyunProfile, AwsProfile, HuaweiProfile, HuaweiCredential
from asset.utils import to_hump_underline, get_tencent_account_id, get_aliyun_account_id, get_aws_account_id, \
    tencent_parser_response, aliyun_parser_response, aws_parser_response, recursive_list, aws_assume_role, \
    get_huawei_account_id, huawei_parser_response, retry

from huaweicloudsdkcore.auth.credentials import BasicCredentials


class Describe:
    def parser_response(self):
        raise NotImplementedError("")

    def describe(self):
        raise NotImplementedError("")


class DescribeTencent(Describe):

    def __init__(
            self,
            client,
            des_request_func: str,
            des_request,
            response_filed: str,
            parser_response: callable = tencent_parser_response
    ):
        self.client = client
        self.des_request_func = des_request_func
        self.des_request = des_request
        self.response_field = response_filed
        self.parser_response_func = parser_response

    @retry(3)
    def parser_response(self):
        return self.parser_response_func(self.describe(), self.response_field)

    def describe(self):
        return getattr(self.client, self.des_request_func)(self.des_request)


class DescribeAliyun(Describe):
    def __init__(
            self,
            client,
            des_request,
            response_filed: str,
            child_response_filed: str = None,
            parser_response_func: callable = aliyun_parser_response
    ):
        self.client = client
        self.des_request = des_request
        self.response_filed = response_filed
        self.child_response_filed = child_response_filed
        self.parser_response_func = parser_response_func

    @retry(5)
    def parser_response(self):
        return self.parser_response_func(self.describe(), self.response_filed, self.child_response_filed)

    def describe(self):
        return getattr(self.client, 'do_action_with_exception')(self.des_request)


class DescribeAws(Describe):
    def __init__(
            self,
            client,
            des_request: str,
            response_field: str,
            child_response_filed: str = None,
            des_request_kwargs: dict = None,
            parser_response_func: callable = aws_parser_response
    ):
        self.client = client
        self.des_request = des_request
        self.des_request_kwargs = dict() if des_request_kwargs is None else des_request_kwargs
        self.response_field = response_field
        self.child_response_filed = child_response_filed
        self.parser_response_func = parser_response_func

    @retry(3)
    def parser_response(self):
        return self.parser_response_func(self.describe(), self.response_field, self.child_response_filed)

    def describe(self):
        return getattr(self.client, self.des_request)(**self.des_request_kwargs)


class DescribeHuawei(Describe):

    def __init__(
            self,
            client: object,
            des: str,
            des_request: object,
            response_field: str,
            parser_response_func: callable = huawei_parser_response
    ):
        self.client = client
        self.des = des
        self.des_request = des_request
        self.response_field = response_field
        self.parser_response_func = parser_response_func

    @retry(3)
    def parser_response(self):
        return self.parser_response_func(self.describe(), self.response_field)

    def describe(self):
        return getattr(self.client, self.des)(self.des_request).to_json_object()


class Asset(metaclass=abc.ABCMeta):
    logger = logging.getLogger('cloud-asset-fetch')

    _platform: str = ''

    _table_name: str = None
    _asset_columns: List[AssetColumn] = None
    _is_hump_underline: bool = True
    _table_args: tuple = tuple()
    _table_kwargs: dict = None
    _default_columns = [
        AssetColumn(name='account_id', type='str', len=128, kwargs={'nullable': False, 'default': ''}),
        AssetColumn(name='region', type='str', len=128, kwargs={'nullable': False, 'default': ''}),
        AssetColumn(
            name='record_date', type='date',
            kwargs={'nullable': False, 'default': datetime.datetime.now, 'onupdate': datetime.datetime.now}
        )
    ]
    _field_document: str = ''

    def __init__(
            self, cred, region=None, dbconfig: DbConfig = None, parser_response: callable = None
    ):
        self.cred = cred
        self.region = region
        self.dbconfig = dbconfig
        self._engine = create_engine(
            "postgresql://{user}:{password}@{host}:{port}/{database}".format(**self.dbconfig.dict())
        )
        if not self._table_name.startswith(f'{self._platform}_'):
            self._table_name = f'{self._platform}_{self._table_name}'
        self.asset_table = AssetTable(
            self._table_name, self._engine, self._asset_columns,
            default_columns=self._default_columns,
            is_hump_underline=self._is_hump_underline,
            args=self._table_args,
            kwargs=self._table_kwargs
        )
        self.parser_response = parser_response
        self._account_id = self._get_account_id()
        for default_column in self._default_columns:
            if default_column.name == 'account_id':
                default_column.kwargs['default'] = self.account_id
            if default_column.name == 'region':
                default_column.kwargs['default'] = self.region

    @property
    def engine(self):
        return self._engine

    @property
    def table(self) -> Table:
        return self.asset_table.table

    @property
    def assets(self) -> list:
        return self._get_assets()

    @property
    def paginate_all_assets(self) -> list:
        return self._paginate_all_assets()

    @property
    def client(self):
        return self._get_client()

    @property
    def account_id(self):
        return self._account_id

    def _paginate_all_assets(self) -> list:
        raise NotImplementedError("")

    def _get_assets(self) -> list:
        raise NotImplementedError("")

    def _get_client(self):
        raise NotImplementedError("")

    def _get_account_id(self):
        raise NotImplementedError("")

    def fetch(self):

        if self._is_hump_underline:
            paginate_all_assets = self.assets_to_hump_underline(self.paginate_all_assets, self.asset_table.columns)
        else:
            paginate_all_assets = self.paginate_all_assets

        # all_assets datetime, bool, ... to str
        paginate_all_assets = recursive_list(paginate_all_assets)
        if paginate_all_assets:
            uc = [_ for _ in self._table_args if isinstance(_, UniqueConstraint)]
            if uc:
                AssetTable.insert_values_duplicat_do_nothing(self.table, paginate_all_assets, self.engine, uc[0].name)
            else:
                AssetTable.insert_values(self.table, paginate_all_assets, self.engine)
        return True

    @classmethod
    def assets_to_hump_underline(cls, assets: List[dict], asset_columns: List[AssetColumn]) -> List[dict]:
        _assets, asset_columns = [], [asset_column.name for asset_column in asset_columns]

        for asset in assets:
            _ = copy.deepcopy(asset)
            _asset = {}
            _asset_keys = {to_hump_underline(key): key for key in _.keys()}
            for asset_column in asset_columns:
                if asset_column not in _asset_keys:
                    if asset_column in ('account_id', 'record_date', 'region'):
                        continue
                    _asset.update({asset_column: None})
                else:
                    _asset[asset_column] = asset.pop(_asset_keys[asset_column])
            _assets.append(_asset)

        return _assets

    @classmethod
    def load_creds(cls, profile):
        raise NotImplementedError("")


class TencentAsset(Asset):
    _platform = 'tencent'

    _describe = DescribeTencent
    _des_request_func: str = ''
    _des_request: object = None
    _response_field: str = ''

    _paginate: bool = True
    _paginate_type = 'int'  # int or str

    def __init__(
            self,
            cred: STSAssumeRoleCredential,
            region=None,
            dbconfig: DbConfig = None,
            parser_response: callable = tencent_parser_response
    ):
        super(TencentAsset, self).__init__(cred, region=region, dbconfig=dbconfig, parser_response=parser_response)
        self.asset_describe = self._describe(
            self.client, self._des_request_func, self._des_request, self._response_field)

    def _get_assets(self):
        return self.asset_describe.parser_response()

    def _paginate_all_assets(self):
        page, assets = 0, []
        _des_request = copy.deepcopy(self._des_request)
        if self._paginate:
            _des_request.Limit = 50 if self._paginate_type == 'int' else '50'

        while True:
            response = self._describe(
                self.client, self._des_request_func, _des_request, self._response_field).parser_response()
            if not response:
                break

            assets += response

            page = 1
            if self._paginate:
                if isinstance(_des_request.Limit, str):
                    _des_request.Offset = str(page * int(_des_request.Limit))
                else:
                    _des_request.Offset = page * _des_request.Limit
            else:
                break

        return assets

    def _get_client(self):
        pass

    def _get_account_id(self):
        return get_tencent_account_id(self.cred)

    @classmethod
    def load_creds(cls, profile: TencentProfile) -> List[STSAssumeRoleCredential]:
        return [
            STSAssumeRoleCredential(
                profile.ak,
                profile.sk,
                role.arn,
                role.session_name,
                duration_seconds=role.duration_seconds
            ) for role in profile.roles
        ]


class AliyunAsset(Asset):
    _platform = 'aliyun'

    _des_request: callable = None
    _response_field = ''
    _child_response_filed = None

    _describe = DescribeAliyun

    _paginate = True

    def __init__(
            self,
            cred: RamRoleArnCredential,
            region: str = None,
            dbconfig: DbConfig = None,
            parser_response: callable = aliyun_parser_response
    ):
        super(AliyunAsset, self).__init__(cred, region=region, dbconfig=dbconfig, parser_response=parser_response)
        self._des_request = self._des_request()
        self._des_request.set_accept_format('json')
        """
                    client,
            des_request,
            response_filed: str,
            parser_response_func: callable = aliyun_parser_response
        """
        self.asset_describe = self._describe(
            self.client,
            self._des_request,
            self._response_field,
            self._child_response_filed,
            self.parser_response
        )

    def _paginate_all_assets(self):
        page, page_size, assets = 0, 50, []
        if self._paginate:
            self._des_request.set_PageSize(page_size)
        while True:
            response = self._describe(
                self.client,
                self._des_request,
                self._response_field,
                self._child_response_filed,
                self.parser_response
            ).parser_response()
            if not response:
                break

            assets += response
            page += 1
            if self._paginate:
                self._des_request.set_PageNumber(page*page_size)
            else:
                break

        return assets

    def _get_client(self):
        return AcsClient(credential=self.cred, region_id=self.region)

    def _get_assets(self):
        return self.asset_describe.parser_response()

    def _get_account_id(self):
        return get_aliyun_account_id(cred=self.cred)

    @classmethod
    def load_creds(cls, profile: AliyunProfile) -> List[RamRoleArnCredential]:
        return [
            RamRoleArnCredential(
                profile.ak,
                profile.sk,
                role.arn,
                role.session_name
            ) for role in profile.roles
        ]


class AwsAsset(Asset):
    _platform = 'aws'

    _client_name: str = ''
    _des_request: str = ''
    _response_field: str = ''
    _child_response_filed: str = None
    _des_request_kwargs: dict = {'MaxResults': 50}
    _next_type = 'NextToken'

    _describe = DescribeAws

    def __init__(
            self,
            cred: AwsCredential,
            region=None,
            dbconfig: DbConfig = None,
            parser_response: callable = aws_parser_response
    ):
        super(AwsAsset, self).__init__(cred, region=region, dbconfig=dbconfig, parser_response=parser_response)
        self.asset_describe = self._describe(
            self.client,
            self._des_request,
            self._response_field,
            des_request_kwargs=self._des_request_kwargs,
            parser_response_func=parser_response
        )

    def _paginate_all_assets(self) -> list:
        assets = []
        _des_request_kwargs = copy.deepcopy(self._des_request_kwargs)
        while True:
            _assets, next_token = self._describe(
                self.client,
                self._des_request,
                self._response_field,
                self._child_response_filed,
                des_request_kwargs=_des_request_kwargs,
                parser_response_func=self.parser_response
            ).parser_response()
            assets += _assets
            if next_token is None:
                break
            _des_request_kwargs.update({self._next_type: next_token})
        return assets

    def _get_client(self):
        """由子类实现"""
        return Session(**self.cred.dict()).client(self._client_name, region_name=self.region)

    def _get_assets(self) -> list:
        assets, _ = self.asset_describe.parser_response()
        return assets

    def _get_account_id(self):
        return get_aws_account_id(self.cred)

    @classmethod
    def load_creds(cls, profile: AwsProfile) -> List[AwsCredential]:
        creds = []
        for role in profile.roles:
            try:
                creds.append(
                    aws_assume_role(role.arn, role_session_name=role.session_name, duration_seconds=role.duration_seconds)
                )
            except Exception as e:
                cls.logger.error(f'aws_assume_role fail, error: {e} ')
                continue
        return creds


class HuaweiAsset(Asset):
    _platform = 'huawei'

    _client = None
    _region_obj = None
    _describe = DescribeHuawei
    _des: str = ''
    _request_obj = None
    _request_pars: dict = {'limit': 50, 'offset': 1}
    _response_field = ''
    _offset = True

    def __init__(
            self,
            cred: HuaweiCredential,
            region=None, dbconfig: DbConfig = None,
            parser_response=huawei_parser_response
    ):
        super(HuaweiAsset, self).__init__(cred, region=region, dbconfig=dbconfig, parser_response=parser_response)

    @property
    def credential(self):
        return BasicCredentials(self.cred.ak, self.cred.sk)

    def _paginate_all_assets(self) -> list:
        assets = []
        _des_request_kwargs = self._request_obj(**self._request_pars)
        while True:
            response = self._describe(
                self.client,
                self._des,
                _des_request_kwargs,
                self._response_field,
                self.parser_response
            ).parser_response()
            assets += response

            if not response:
                break
            if self._offset:
                _des_request_kwargs.offset += 1
            else:
                if _des_request_kwargs.limit:
                    if len(response) < _des_request_kwargs.limit:
                        break
                else:
                    break

        return assets

    def _get_assets(self) -> list:
        pass

    def _get_account_id(self):
        return get_huawei_account_id(self.credential)

    def _get_client(self):
        region = self._region_obj.value_of(self.region)
        return getattr(
            getattr(self._client.new_builder(), 'with_credentials')(self.credential),
            'with_region'
        )(region).build()

    @classmethod
    def load_creds(cls, profile: HuaweiProfile) -> List[HuaweiCredential]:
        return profile.credentials



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import AliyunAsset, AssetColumn, UniqueConstraint
from aliyunsdkram.request.v20150501.ListUsersRequest import ListUsersRequest
from aliyunsdkram.request.v20150501.GetUserRequest import GetUserRequest
from aliyunsdkram.request.v20150501.GetLoginProfileRequest import GetLoginProfileRequest
from aliyunsdkram.request.v20150501.ListAccessKeysRequest import ListAccessKeysRequest


ram_asset_columns = [
    {'name': 'DisplayName', 'type': 'str'},
    {'name': 'Email', 'type': 'str'},
    {'name': 'UpdateDate', 'type': 'str'},
    {'name': 'MobilePhone', 'type': 'str'},
    {'name': 'UserId', 'type': 'str'},
    {'name': 'Comments', 'type': 'str'},
    {'name': 'CreateDate', 'type': 'str'},
    {'name': 'UserName', 'type': 'str'},
    {'name': 'LastLoginDate', 'type': 'dict'},
    {'name': 'LoginProfile', 'type': 'dict'},
    {'name': 'AccessKeys', 'type': 'list'},
]

ram_field_document = 'https://next.api.alibabacloud.com/document/Ram/2015-05-01/ListUsers'


@cloud_providers.aliyun.register('user')
class User(AliyunAsset):
    _des_request = ListUsersRequest
    _response_field = 'Users'
    _child_response_filed = 'User'

    _paginate = False

    _table_name = 'aliyun_user'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in ram_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'user_id', name='aliyun_uc_user'),)
    _field_document = ram_field_document

    def _paginate_all_assets(self):
        assets = super()._paginate_all_assets()
        if not assets:
            return assets

        for asset in assets:
            request = GetUserRequest()
            request.set_UserName(asset['UserName'])
            last_login = self._describe(self.client, request, 'User').parser_response()
            if last_login:
                asset.update({'LastLoginDate': last_login['LastLoginDate']})

            # get loging profile
            last_login_request = GetLoginProfileRequest()
            last_login_request.set_UserName(asset['UserName'])
            login_profile = self._describe(self.client, last_login_request, 'LoginProfile').parser_response()

            if login_profile:
                asset.update({'LoginProfile': login_profile})

            # get access keys
            access_keys_request = ListAccessKeysRequest()
            access_keys_request.set_UserName(asset['UserName'])
            access_keys = self._describe(self.client, access_keys_request, 'AccessKeys').parser_response()
            if access_keys:
                asset.update({'AccessKeys': access_keys})

        return assets



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import TencentAsset, AssetColumn, UniqueConstraint
from qcloud_cos import CosConfig, CosS3Client

cos_asset_columns = [
    {'name': 'Name', 'type': 'str'},
    {'name': 'Location', 'type': 'str'},
    {'name': 'CreationDate', 'type': 'str'},
    {'name': 'OwnerDisplayName', 'type': 'str'},
    {'name': 'OwnerId', 'type': 'str'},
]

cos_field_document = 'https://cloud.tencent.com/document/product/436/35150'


@cloud_providers.tencent.register('cos')
class Cos(TencentAsset):
    _des_request_func = 'list_buckets'
    _response_field = 'Buckets'

    _table_name = 'tencent_cos'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in cos_asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'name', name='tencent_uc_cos'),)
    _field_document = cos_field_document

    def _paginate_all_assets(self):
        assets = []
        config = CosConfig(
            Region=self.region, SecretId=self.cred.secretId, SecretKey=self.cred.secretKey, Token=self.cred.token)
        client = CosS3Client(config)
        response = client.list_buckets()

        owner = response.get('Owner', {})
        bucket_dict = response.get('Buckets', {})

        if not bucket_dict:
            return assets

        for asset in bucket_dict.get('Bucket', []):
            asset.update(
                {
                    'OwnerDisplayName': owner.get('DisplayName', ''),
                    'OwnerId': owner.get('ID', '')
                }
            )
            assets.append(asset)
        return assets

    def _get_client(self):
        pass



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from asset.asset_register import cloud_providers
from asset.base import AwsAsset, AssetColumn, UniqueConstraint

alternate_contact_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/account/client/get_alternate_contact.html'
contact_information_field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/account/client/get_contact_information.html'

alternate_contact_columns = [
    {'name': 'AlternateContactType', 'type': 'str'},
    {'name': 'EmailAddress', 'type': 'str'},
    {'name': 'Name', 'type': 'str'},
    {'name': 'PhoneNumber', 'type': 'str'},
    {'name': 'Title', 'type': 'str'}
]
contact_information_columns = [
    {'name': 'AddressLine1', 'type': 'str'},
    {'name': 'AddressLine2', 'type': 'str'},
    {'name': 'AddressLine3', 'type': 'str'},
    {'name': 'City', 'type': 'str'},
    {'name': 'CompanyName', 'type': 'str'},
    {'name': 'CountryCode', 'type': 'str'},
    {'name': 'DistrictOrCounty', 'type': 'str'},
    {'name': 'FullName', 'type': 'str'},
    {'name': 'PhoneNumber', 'type': 'str'},
    {'name': 'PostalCode', 'type': 'str'},
    {'name': 'StateOrRegion', 'type': 'str'},
    {'name': 'WebsiteUrl', 'type': 'str'}
]


@cloud_providers.aws.register('contact_information')
class ContactInformation(AwsAsset):
    _client_name = 'account'
    _des_request = 'get_contact_information'
    _response_field = 'ContactInformation'

    _des_request_kwargs = {}

    _table_name = 'aws_contact_information'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in contact_information_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'phone_number', name='aws_uc_account_ci'),)
    _field_document = contact_information_field_document


@cloud_providers.aws.register('alternate_contact')
class AlternateContact(AwsAsset):
    _client_name = 'account'
    _des_request = 'get_alternate_contact'
    _response_field = 'AlternateContact'
    _des_request_kwargs = {'AlternateContactType': 'OPERATIONS'}

    _table_name = 'aws_alternate_contact'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in alternate_contact_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'phone_number', name='aws_uc_account_ac'),)
    _field_document = alternate_contact_field_document
